//------------------------------------------------------------------------------
/*
    This file is part of Bottle2D.

    Bottle2D is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Bottle2D is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bottle2D.  If not, see <http://www.gnu.org/licenses/>.
*/
//------------------------------------------------------------------------------
#ifndef MY_EVENT_HPP
#define MY_EVENT_HPP

#include <SFML/Graphics.hpp>
#include "Object.hpp"

#include <string>
#include <map>

#include "ClassAllocCounter.hpp"
//------------------------------------------------------------------------------
#include <luabind/object.hpp>
//------------------------------------------------------------------------------
namespace En
{
class Screen;
class MouseZoneEvent;
//------------------------------------------------------------------------------
/// LUA_HEADER_COPY_ON ///
/** Base event handler that can be destroyed. */
/// LUA_HEADER_COPY_OFF ///
class EventHandlerBase : public ClassAllocCounter
{
public:
    EventHandlerBase(const luabind::object &luaFunction);

    /// LUA_EXPORT_ON(EventHandlerBase) ///
    // Lua
    // Destroy the object (it will be destroyed at the end of the frame)
    void Destroy();

    // Check if the event will be destroyed at the end of the frame
    bool ShouldBeDestroyed() const {return m_isEndOfLife;}

    // User data
    /** Set user data */
    void SetUserData(const std::string &id, const luabind::object &data);
    /** Get user data */
    luabind::object GetUserData(const std::string &id);
    /** Return true if the user data exists, false otherwise */
    bool UserDataExists(const std::string &id);

    /// LUA_EXPORT_OFF ///

protected:
    std::map<std::string, luabind::object> m_userData;

    luabind::object m_luaFunction;
    bool m_isEndOfLife;
};
//------------------------------------------------------------------------------
bool IsKeyPressed(const std::string &keyName, const sf::Input &input);
//------------------------------------------------------------------------------
class EventHandler : public EventHandlerBase
{
public:
    EventHandler(const luabind::object &luaFunction);
    virtual bool Match(const sf::Event &event, const Screen *screen, const sf::RenderWindow &App) = 0;
};
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
class ButtonEventHandler : public EventHandler
{
public:
    ButtonEventHandler(const luabind::object &luaFunction, const std::string &keyName = "", bool pressed = true);
    virtual bool Match(const sf::Event &event, const Screen *screen, const sf::RenderWindow &App);

    void SetEventFunction(void(*evtFn)(const std::string &, bool, ButtonEventHandler*, const luabind::object &));

    static void FillMaps();

    static int KeyNameToId(const std::string &name);
    static std::string KeyIdToName(int id);

    std::string GetKeyName() const {return m_keyName;}
    bool GetPressed() const {return m_pressed;}


protected:
    bool ParseEvent(const sf::Event &event, std::string &keyName, bool &pressed);
    static void SendEvent(const std::string &keyName, bool pressed, ButtonEventHandler *obj, const luabind::object &luaFunction);

    bool m_pressed;
    std::string m_keyName;
    int m_keyId;

    void(*m_evtFn)(const std::string &, bool, ButtonEventHandler*, const luabind::object &);

    static std::map<int, std::string> m_mapIdToName;
    static std::map<std::string, int> m_mapNameToId;
};
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
class ButtonInZoneEventHandler : public ButtonEventHandler
{
public:
    ButtonInZoneEventHandler(const luabind::object &luaFunction, MouseZoneEvent *zoneEvt, const std::string &keyName = "", bool pressed = true);
    virtual bool Match(const sf::Event &event, const Screen *screen, const sf::RenderWindow &App);

protected:
    static void SendEvent(const std::string &keyName, bool pressed, MouseZoneEvent *zoneEvtObj, ButtonInZoneEventHandler *obj, const luabind::object &luaFunction);

    MouseZoneEvent *m_zoneEvt;
};
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
class MouseMoveEventHandler : public EventHandler
{
public:
    MouseMoveEventHandler(const luabind::object &luaFunction);
    virtual bool Match(const sf::Event &event, const Screen *screen, const sf::RenderWindow &App);

private:
    static void SendEvent(const Vector2D &pos, MouseMoveEventHandler *obj, const luabind::object &luaFunction);
};
//------------------------------------------------------------------------------
} // namespace En
#endif // MY_EVENT_HPP
