//------------------------------------------------------------------------------
/*
    This file is part of Bottle2D.

    Bottle2D is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Bottle2D is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bottle2D.  If not, see <http://www.gnu.org/licenses/>.
*/
//------------------------------------------------------------------------------
#ifndef PHCIRCLE_HPP
#define PHCIRCLE_HPP
//------------------------------------------------------------------------------
#include "PhysicObject.hpp"
//------------------------------------------------------------------------------
namespace En
{
//------------------------------------------------------------------------------
/// LUA_HEADER_COPY_ON ///
/** A physic object representing a circle / disk created using Group::CreatePhCircle. */
/// LUA_HEADER_COPY_OFF ///
//------------------------------------------------------------------------------
class PhCircle : public PhysicObject
{
public:
    PhCircle(Group *parent, int phGroupId, const std::string &name, const Vector2D &center, float radius);
    static PhCircle *Create(Group *parent, int phGroupId, const std::string &name, const Vector2D &center, float radius) {return new PhCircle(parent, phGroupId, name, center, radius);}
    virtual ~PhCircle();

    /// LUA_EXPORT_ON(PhCircle, PhysicObject) ///
    // Radius
    /** Access circle's radius */
    //LUA// luaProperty(float, radius, GetRadius, SetRadius);
    /** Set circle's radius */
    void SetRadius(float radius);
    /** Get circle's radius */
    float GetRadius() const;

    // Clone & Create
    /** Create a clone of the object and return an handler to the clone. */
    virtual PhCircle *Clone() const {return CloneToGroup(NULL);}

    /** Create a clone of the object, change the parent and return an handler to the clone. */
    virtual PhCircle *CloneToGroup(Group *newParent) const {PhCircle *obj = new PhCircle(*this); obj->RegisterClone(newParent); return obj;}
    /// LUA_EXPORT_OFF ///

    // No Lua
    // Lua
    //static void RegisterClassLua(lua_State *myLuaState);

    virtual bool IsPointInside(const Vector2D &point);

    // Type
    ADD_GET_TYPE_FN(OBJ_TYPE_PHYSIC_CIRCLE)

    // All pts
    virtual void RenderShape(sf::RenderTarget &App) const;

    // Maximum interaction radius
    virtual float GetMaxInteractionRadius() const {return m_radius;}

protected:
    float m_radius;

private:
};
//------------------------------------------------------------------------------
} // namespace En
//------------------------------------------------------------------------------
#endif // PHCIRCLE_HPP
