//------------------------------------------------------------------------------
/*
    This file is part of Bottle2D.

    Bottle2D is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Bottle2D is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bottle2D.  If not, see <http://www.gnu.org/licenses/>.
*/
//------------------------------------------------------------------------------
#include "PhysicObject.hpp"

#include "Group.hpp"
#include "Screen.hpp"
#include "PhCircle.hpp"

#include "MyException.hpp"

#include <lua.hpp>
#include <luabind/luabind.hpp>

#include <iostream>
//------------------------------------------------------------------------------
namespace En
{
using namespace std;

LinePtVect::LinePtVect(const Vector2D &cPt, const Vector2D &cVect) : pt(cPt), vect(cVect) {}
LinePtVect::LinePtVect(const LinePtPt &line) : pt(line.pt1), vect(line.pt2 - line.pt1) {}
float LinePtVect::GetSize() const {return sqrt(GetSqrSum());};
float LinePtVect::GetSqrSum() const {return vect.x * vect.x + vect.y * vect.y;};

LinePtPt::LinePtPt(const Vector2D &cPt1, const Vector2D &cPt2) : pt1(cPt1), pt2(cPt2) {}
LinePtPt::LinePtPt(const LinePtVect &line) : pt1(line.pt), pt2(line.pt + line.vect) {}
float LinePtPt::GetSize() const {return sqrt(GetSqrSum());};
float LinePtPt::GetSqrSum() const {return LinePtVect(*this).GetSqrSum();};

//------------------------------------------------------------------------------
PhysicObject::PhysicObject(Group *parent, int phGroupId, const std::string &name) : Object(parent, name), m_group(phGroupId)
{
    m_parent = NULL;
    ChangeParent(parent);
}
//------------------------------------------------------------------------------
/*PhysicObject::PhysicObject(const PhysicObject& src) : Object(src)
{
    m_group = src.m_group;
    m_fnName = src.m_fnName;
    GetParent()->Add(this);
    GetParentScreen()->GetPhysicWorld()->Attach(this);
}*/
//------------------------------------------------------------------------------
void PhysicObject::RegisterClone(Group *newParent)
{
    // We change the parent if it's needed
    if(newParent != NULL)
    {
        m_parent = newParent;
    }

    // We register the object
    GetParent()->Add(this);

    // We notice the object that the parent has changed => Will attach the objet to the pysic world
    OnChangeScreen(NULL, GetParentScreen());
}
//------------------------------------------------------------------------------
/*void PhysicObject::SetGroup(int grp)
{
    m_group = grp;
}*/
//------------------------------------------------------------------------------
int PhysicObject::GetPhysicGroup() const
{
    return m_group;
}
//------------------------------------------------------------------------------
PhysicObject::~PhysicObject()
{

}
//------------------------------------------------------------------------------
void PhysicObject::OnIntersect(PhysicObject *obj1, PhysicObject *obj2, const Vector2D &pos)
{
    const luabind::object &luaFunction = m_luaFunction;

    if(! luaFunction.is_valid())
    {
        return;
    }

    try
    {
        luabind::call_function<void>(luaFunction, obj1, obj2, pos);
    }
    MY_CATCH_LUA_CERR()
}
//------------------------------------------------------------------------------
void PhysicObject::SetEventFunction(const luabind::object &luaFunction)
{
    if(luaFunction.is_valid() && luabind::type(luaFunction) == LUA_TFUNCTION)
    {
        m_luaFunction = luaFunction;
    }
    else
    {
        m_luaFunction = luabind::object();
    }
}
//------------------------------------------------------------------------------
bool PhysicObject::Intersect(const PhysicObject &obj, Vector2D &pos) const
{
    // FIXME : Optimisation
    // -> Cache line list ?

    // Max interaction distance
    if(LinePtPt(GetGlobalPosition(), obj.GetGlobalPosition()).GetSize() > (GetMaxInteractionRadius() + obj.GetMaxInteractionRadius()))
    {
        return false;
    }

    // Test
    if(obj.GetType() != OBJ_TYPE_PHYSIC_CIRCLE && GetType() != OBJ_TYPE_PHYSIC_CIRCLE)
    {
        const PhysicObjectLine &obj1 = static_cast<const PhysicObjectLine&>(obj);
        const PhysicObjectLine &obj2 = static_cast<const PhysicObjectLine&>(*this);

        std::vector<Vector2D> list1 = obj1.GetAllPoints();
        std::vector<Vector2D> list2 = obj2.GetAllPoints();

        for(size_t i = 0; i < list1.size(); i++)
        {
            for(size_t j = 0; j < list2.size(); j++)
            {
                LinePtPt line1(list1[i], list1[(i + 1) % list1.size()]);
                LinePtPt line2(list2[j], list2[(j + 1) % list2.size()]);

                if(IsLineInter(line1, line2, pos))
                {
                    return true;
                }
            }
        }
    }
    else
    {
        const PhCircle *circle1;
        const PhysicObject *obj2;
        if(obj.GetType() == OBJ_TYPE_PHYSIC_CIRCLE)
        {
            circle1 = static_cast<const PhCircle*>(&obj);
            obj2 = this;
        }
        else
        {
            circle1 = static_cast<const PhCircle*>(this);
            obj2 = &obj;
        }

        if(obj2->GetType() == OBJ_TYPE_PHYSIC_CIRCLE) // Circle * 2
        {
            const PhCircle *circle2 = static_cast<const PhCircle*>(obj2);

            LinePtPt line(circle1->GetGlobalPosition(), circle2->GetGlobalPosition());
            float sqrSum = line.GetSqrSum();

            bool result = sqrSum <= pow(circle1->GetRadius() + circle2->GetRadius(), 2);

            if(result)
            {
                pos = line.pt1 + (line.pt2 - line.pt1) * ((circle1->GetRadius() + circle2->GetRadius()) / circle1->GetRadius());
            }

            return result;
        }
        else // Circle + shape
        {
            const PhysicObjectLine &objLine2 = static_cast<const PhysicObjectLine&>(*obj2);

            std::vector<Vector2D> list1 = objLine2.GetAllPoints();

            for(size_t i = 0; i < list1.size(); i++)
            {
                LinePtPt line1(list1[i], list1[(i + 1) % list1.size()]);

                Vector2D circlePos = circle1->GetGlobalPosition();
                if(IsLineCircleInter(line1, circlePos, circle1->GetRadius(), pos))
                {
                    return true;
                }
            }
        }
    }


    return false;
}
//------------------------------------------------------------------------------
void PhysicObject::ChangeParent(Group *parent)
{
    // /!\ à copier pour PhysicObject quant modifié !!
    Screen *backScreen = NULL, *newScreen = NULL;

    if(m_parent != NULL)
    {
        backScreen = GetParentScreen();
        m_parent->Remove(this);
    }

    // m_parent = parent; // If parent is NULL, the object wille be deleted and it may need to know its parent

    if(parent != NULL)
    {
        m_parent = parent;
        parent->Add(this);
        newScreen = GetParentScreen();
    }

    OnChangeScreen(backScreen, newScreen);
}
//------------------------------------------------------------------------------
void PhysicObject::OnChangeScreen(Screen *backScreen, Screen *newScreen)
{
    if(backScreen != newScreen) // If we don't change screen, this is unusefull
    {
        if(backScreen != NULL)
        {
            backScreen->GetPhysicWorld()->Detach(this);
        }

        if(newScreen != NULL)
        {
            newScreen->GetPhysicWorld()->Attach(this);
        }
    }
}
//------------------------------------------------------------------------------
bool PhysicObject::IsLineInter(const LinePtVect &l1, const LinePtVect &l2, Vector2D &pos) const
{
    // Cf http://www.siteduzero.com/forum-83-382906-p1-calcul-du-point-d-intersection-de-deux-segments.html

    const Vector2D &A = l1.pt;
    const Vector2D &C = l2.pt;

    const Vector2D &I = l1.vect;
    const Vector2D &J = l2.vect;

    const float div = I.x * J.y - I.y * J.x;
    if(div == 0) // Parallele !
    {
        return false;
    }
    else
    {
        float ratio1 = (I.x * A.y - I.x * C.y - I.y * A.x + I.y * C.x) / div;
        float ratio2 = (J.x * A.y - J.x * C.y - J.y * A.x + J.y * C.x) / div;

        if((0 <= ratio1 && ratio1 <= 1) && (0 <= ratio2 && ratio2 <= 1))
        {
            pos = A + I * ratio2;

            return true;
        }
        else
        {
            return false;
        }
    }
}
//------------------------------------------------------------------------------
luabind::object PhysicObject::GetEventFunction() const
{
    return m_luaFunction;
}
//------------------------------------------------------------------------------
bool PhysicObject::IsLineCircleInter(const LinePtVect &l1, const Vector2D &center, float radius, Vector2D &pos) const
{
    const Vector2D &A = l1.pt; // Line 1 : start point
    const Vector2D &I = l1.vect; // Line 1 : Vect

    const Vector2D &C = center; // Circle center

    float ratio = -(A.x*I.x - C.x*I.x + A.y*I.y - C.y*I.y) / (pow(I.x, 2) + pow(I.y, 2));

    // On the line
    ratio = ratio > 1.f ? 1.f : ratio;
    ratio = ratio < 0.f ? 0.f : ratio;

    float squareDist = pow(A.x - C.x + ratio * I.x, 2) + pow(A.y - C.y + ratio * I.y, 2);
    bool result = (squareDist <= pow(radius, 2));

    if(result)
    {
        pos = A + I * ratio;
    }

    return(result);
}
//------------------------------------------------------------------------------
/*void PhysicObject::RegisterClassLua(lua_State *myLuaState)
{
    luabind::module(myLuaState)
    [
        luabind::class_<Object>("Object")

    ];
}*/
//------------------------------------------------------------------------------
void PhysicObjectLine::RenderShape(sf::RenderTarget &App) const
{
    vector<Vector2D> ptList = GetAllPoints();
    for(size_t j = 0; j < ptList.size(); j++)
    {
        LinePtPt line(ptList[j], ptList[(j + 1) % ptList.size()]);
        sf::Shape sfLine = sf::Shape::Line(line.pt1.x, line.pt1.y, line.pt2.x, line.pt2.y, 1, Color(0, 0, 255));
        App.Draw(sfLine);

        sf::Shape circle = sf::Shape::Circle(ptList[j].x, ptList[j].y, 4, Color(255, 255, 255));
        App.Draw(circle);
    }
}
//------------------------------------------------------------------------------
bool PhysicObjectLine::IsPointInside(const Vector2D &point)
{
    if(LinePtPt(GetGlobalPosition(), point).GetSize() > GetMaxInteractionRadius())
    {
        return false;
    }

    // Ray
    bool crossing = false; // even / pair
    Vector2D pos(0, 0);

    std::vector<Vector2D> list1 = GetAllPoints();

    Vector2D rayStart = GetGlobalPosition() + Vector2D(m_cachedMaxInterRadius, m_cachedMaxInterRadius);
    LinePtPt ray(rayStart, point);

    for(size_t i = 0; i < list1.size(); i++)
    {
        LinePtPt line(list1[i], list1[(i + 1) % list1.size()]);

        if(IsLineInter(line, ray, pos))
        {
            crossing = !crossing;
        }
    }

    return crossing;
}
//------------------------------------------------------------------------------
void PhysicObjectLine::RefreshMaxInterRadius()
{
    m_cachedMaxInterRadius = 0;

    Vector2D pos = GetGlobalPosition();
    std::vector<Vector2D> list = GetAllPoints();

    for(std::vector<Vector2D>::iterator it = list.begin(); it != list.end(); it++)
    {
        m_cachedMaxInterRadius = max(LinePtPt(pos, *it).GetSize(), m_cachedMaxInterRadius);
    }
}

//------------------------------------------------------------------------------
} // namespace En
//------------------------------------------------------------------------------
