require("guiLib/common")

------------------------------------------------------------
-- Params
------------------------------------------------------------

currentMaxZPos = 1000 -- The zpos of the current top window

windowMargin = 15
scrollMargin = 45

------------------------------------------------------------
-- Create a window
------------------------------------------------------------
function windowInit(group, isBorder)
	-- Values
	group:SetUserData("C501_windowIdentifier", nil) -- To check if the group is a window
	group:SetUserData("isMoving", false)
	group:SetUserData("isScrolling", false)
	group:SetUserData("isAlwaysActive", false)
	group:SetUserData("isActive", true)
	
	-- Colors
	--group:SetUserData("backColorNormal", Color(140, 71, 0, 255))
	--group:SetUserData("backColorActive", Color(249, 141, 0, 255))
	
	-- Bg
	group:SetUserData("backColorNormal", windowNormalBgTex)
	group:SetUserData("backColorActive", windowActiveBgTex)	
	
	-- Border
	group:SetUserData("isBorder", isBorder)
	

	-- Border, 5000 < Zpos < 6000
	if isBorder == true then
		local tmp = group:CreateSprite("bottomLeft")
		tmp:SetImage(bottomLeftTex)
		tmp:SetZpos(5100)
		local tmp = group:CreateSprite("bottomLine")
		tmp:SetImage(bottomLineTex)
		tmp:SetZpos(5100)
		local tmp = group:CreateSprite("bottomRight")
		tmp:SetImage(bottomRightTex)
		tmp:SetZpos(5100)
		local tmp = group:CreateSprite("leftLine")
		tmp:SetImage(leftLineTex)
		tmp:SetZpos(5100)
		local tmp = group:CreateSprite("rightLine")
		tmp:SetImage(rightLineTex)
		tmp:SetZpos(5100)
		local tmp = group:CreateSprite("topLeft")
		tmp:SetImage(topLeftTex)
		tmp:SetZpos(5100)
		local tmp = group:CreateSprite("topLine")
		tmp:SetImage(topLineTex)
		tmp:SetZpos(5100)
		local tmp = group:CreateSprite("topRight")
		tmp:SetImage(topRightTex)
		tmp:SetZpos(5100)

		local closeBtXPos = 42

		-- Title, 6000 < Zpos < 7000
		local tmp = group:CreateSprite("titleBar")
		tmp:SetImage(titleTex)
		tmp:SetPosition(Vector2D(30, topLineTex:GetHeight() - titleTex:GetHeight()) + Vector2D(0.375, 0.375))
		tmp:SetZpos(6100)
		tmp:Show(false)
		
		local tmpZone = group:CreatePhRect(0, "windowZoneBis", tmp:GetSize())
		tmpZone:SetPosition(tmp:GetPosition())
	
		local tmpZone = group:CreatePhRect(0, "moveZone", tmp:GetSize() - Vector2D(closeBtXPos + closeBtNormalTex:GetWidth() + 6, 0))
		tmpZone:SetPosition(Vector2D(closeBtXPos + closeBtNormalTex:GetWidth() + 5, tmp:GetPosition().y))

		-- Close bt
		local tmp = group:CreateGroup("closeBt")
		buttonControlCreate(tmp, "", closeBtNormalTex:GetSize())
		buttonSetImagePack(tmp, closeBtNormalTex, closeBtOverTex, closeBtPressedTex)
		tmp:SetPosition(Vector2D(closeBtXPos, topLineTex:GetHeight() - (titleTex:GetHeight() + closeBtNormalTex:GetHeight()) / 2 + 9))
		tmp:SetZpos(6300)
		
		local closeBtSpr = tmp

		local tmp = group:CreateText("titleText", "A")
		tmp:SetSize(30)
		tmp:SetPosition(Vector2D(closeBtSpr:GetPosition().x + closeBtOverTex:GetSize().x + 6, closeBtSpr:GetPosition().y + closeBtOverTex:GetSize().y / 2 - 15 - 5))
		tmp:SetColor(Color(0, 0, 0))
		tmp:SetZpos(6200)
		tmp:SetText("")
		tmp:Show(false)
	end
	
	-- Background (Zpos < -5000)
	--[[local tmp = group:CreateShape("background")
	tmp:MakeRectangle(Vector2D(0, 0), Vector2D(10, 10), Color(0, 0, 0), 0, Color(0, 0, 0)) -- Init as a rectangle]]
	local tmp = group:CreateSprite("background")
	tmp:SetZpos(-5001)
	
	-- Window zone (zone for the whole window)
	group:CreatePhRect(0, "windowZone", Vector2D(0, 0))


	-- Black strips (scroll) => 2500 < Zpos < 5000
	--[[local tmp = group:CreateShape("stripTop")
	tmp:SetZpos(2510)
	local tmp = group:CreateShape("stripBottom")
	tmp:SetZpos(2510)]]

	-- Scrollbar => 2500 < Zpos < 5000
	local tmp = group:CreateSprite("arrowDown")
	tmp:SetImage(arrowDownTex)
	tmp:SetZpos(3000)
	group:CreatePhRect(0, "arrowDownZone", tmp:GetSize())
	
	local tmp = group:CreateSprite("arrowUp")
	tmp:SetImage(arrowUpTex)
	tmp:SetZpos(3000)
	group:CreatePhRect(0, "arrowUpZone", tmp:GetSize())

	local tmp = group:CreateSprite("scrollBar")
	tmp:SetImage(scrollBarTex)
	tmp:SetZpos(3400)
	local tmpZone = group:CreatePhRect(0, "scrollBarZone", tmp:GetSize())
	tmpZone:SetUserData("isScrolling", false)
	
	local tmp = group:CreateSprite("scrollBarStart")
	tmp:SetImage(scrollBorderTex)
	tmp:SetZpos(3200)
	
	local tmp = group:CreateSprite("scrollBarEnd")
	tmp:SetImage(scrollBorderTex)
	tmp:SetZpos(3200)
	
	local tmp = group:CreateShape("scrollBarBack")
	tmp:SetZpos(3300)
		
	-- Body position
	local body = group:CreateGroup("body")
	local pos = Vector2D(0, 0) -- No border => No margin !

	if isBorder == true then
		pos = Vector2D(windowMargin + leftLineTex:GetWidth(), windowMargin + topLineTex:GetHeight())
	end
	
	body:SetPosition(pos)
	body:SetUserData("scrollMin", body:GetPosition().y)
	body:SetUserData("scrollHeight", 0)
	
	-- Events
	local screenOfGroup = group:GetParentScreen()
	
		--local eventTbl = {} -- We stock the table of event to remove them when the window is destroyed
	
		-- On frame
		local evt = screenOfGroup:RegisterFrameEvent(windowApi_OnFrame)
		evt:SetUserData("window", group)
		group:RegisterEventAutoRemoveOnDestroy(evt)

		-- Event wheel down
		local evt = screenOfGroup:RegisterButtonEvent(windowApi_OnWheel, "MouseWheelDown", true)
		evt:SetUserData("window", group)
		group:RegisterEventAutoRemoveOnDestroy(evt)
		local evt = screenOfGroup:RegisterButtonEvent(windowApi_OnWheel, "MouseWheelUp", true)
		evt:SetUserData("window", group)
		group:RegisterEventAutoRemoveOnDestroy(evt)

		-- Click on scroll bar
		local zoneEvent = screenOfGroup:RegisterMouseZoneEvent(nil, group:FindByNamePhRect("scrollBarZone"))
		group:RegisterEventAutoRemoveOnDestroy(zoneEvent)
		local evt = screenOfGroup:RegisterButtonInZoneEvent(windowApi_OnScrollBarClick, zoneEvent, "MouseLeft", true)
		group:RegisterEventAutoRemoveOnDestroy(evt)
		
		
		if isBorder == true then		
			-- Title bar move
			local zoneEvent = screenOfGroup:RegisterMouseZoneEvent(nil, group:FindByNamePhRect("moveZone"))
			group:RegisterEventAutoRemoveOnDestroy(zoneEvent)
			local evt = screenOfGroup:RegisterButtonInZoneEvent(windowApi_OnMoveWindow, zoneEvent, "MouseLeft", true)
			group:RegisterEventAutoRemoveOnDestroy(evt)
			
			-- Title bar click => Active window
			local zoneEvent = screenOfGroup:RegisterMouseZoneEvent(nil, group:FindByNamePhRect("windowZoneBis"))
			group:RegisterEventAutoRemoveOnDestroy(zoneEvent)
			local evt = screenOfGroup:RegisterButtonInZoneEvent(windowApi_OnClickInWindow, zoneEvent, "MouseLeft", true)
			group:RegisterEventAutoRemoveOnDestroy(evt)
			
			-- Click on close bt => Close (hide) the window
			buttonSetEventFunction(group:FindByNameGroup("closeBt"), windowApi_OnCloseClick)
		end

		-- Click in the window
		local zoneEvent = screenOfGroup:RegisterMouseZoneEvent(nil, group:FindByNamePhRect("windowZone"))
		group:RegisterEventAutoRemoveOnDestroy(zoneEvent)
		local evt = screenOfGroup:RegisterButtonInZoneEvent(windowApi_OnClickInWindow, zoneEvent, "MouseLeft", true)
		group:RegisterEventAutoRemoveOnDestroy(evt)

		-- Mouse move
		local evt = screenOfGroup:RegisterMouseMoveEvent(windowApi_OnMouseMove)
		evt:SetUserData("window", group)
		group:RegisterEventAutoRemoveOnDestroy(evt)

		-- Event mouse release
		local evt = screenOfGroup:RegisterButtonEvent(windowApi_OnMouseRelease, "MouseLeft", false)
		evt:SetUserData("window", group)
		group:RegisterEventAutoRemoveOnDestroy(evt)
		
	-- Save event list
	--group:SetUserData("eventTbl", eventTbl)
		
	-- Register
	if not screenOfGroup:UserDataExists("windowTbl") then
		screenOfGroup:SetUserData("windowTbl", {})
	end
	
	
	table.insert(screenOfGroup:GetUserData("windowTbl"), group)
	
	-- Make top
	windowMakeTop(group)
end

------------------------------------------------------------
-- Set the close event function (default : hide the window)
------------------------------------------------------------
function windowSetCloseEventFunction(group, fn)
	if fn == nil then
		buttonSetEventFunction(group:FindByNameGroup("closeBt"), windowApi_OnCloseClick)
	else
		buttonSetEventFunction(group:FindByNameGroup("closeBt"), fn)
	end
end

------------------------------------------------------------
-- Remove a window and cleat its events
------------------------------------------------------------
function windowDestroy(group)
	local windowTbl = group:GetParentScreen():GetUserData("windowTbl")

	-- Unregister
	for i = 1, #windowTbl do
		if group == windowTbl[i] then
			table.remove(windowTbl, i)
		end
	end
	
	-- Destroy
	group:Destroy()
	
	-- Set focus to the to level window
	-- We find the window under this one
	local maxId = -1
	
	for i = 1, #windowTbl do
		if (maxId == -1 or windowTbl[i]:IsUpperThan(windowTbl[maxId])) and windowIsVisible(windowTbl[i]) and not windowIsAlwaysActive(windowTbl[i]) then
			maxId = i
		end
	end
	
	-- If there are other window
	if maxId ~= -1 then
		-- We make the window under this one the top window
		windowMakeTop(windowTbl[maxId])
	end
end

------------------------------------------------------------
-- Set the scroll height of a window
------------------------------------------------------------
function windowSetScrollHeight(group, height)
	local body = group:FindByNameGroup("body")
	
	height = math.max(height - group:GetUserData("inSize").y, 0)
	
	body:SetUserData("scrollHeight", height)
	windowScroll(windowGrp, 0)
end

------------------------------------------------------------
-- Resize a window
------------------------------------------------------------
function windowResize(group, position, size, isScroll, isInnerSize)
	-- Position
	group:SetPosition(position)
	
	-- Size
	local vLineWidth = leftLineTex:GetWidth()
	local hLineHeight = topLineTex:GetHeight()
	local stripMargin = hLineHeight + windowMargin
	
	-- Space lost (horizontaly and verticaly)
	local hLost = 0
	local vLost = 0
	if group:GetUserData("isBorder") == true then
		-- Images size
		hLost = hLost + stripMargin * 2
		vLost = vLost + (vLineWidth + windowMargin) * 2
	end
	
	if isScroll == true then
		-- Scroll size
		hLost = hLost + scrollMargin
	end
	
	-- If the size is the inner size
	if isInnerSize then
		size = size + Vector2D(hLost, vLost)
	end
	
	-- Borders
	if group:GetUserData("isBorder") == true then
		-- Images size
		local borderSize = bottomLeftTex:GetSize()
	
		-- Border & corner
		local tmp = group:FindByNameSprite("topLeft")
		tmp:SetPosition(Vector2D(0, 0))
	
		local tmp = group:FindByNameSprite("leftLine")
		tmp:SetPosition(Vector2D(0, borderSize.y))
		tmp:SetSize(Vector2D(vLineWidth, size.y - borderSize.y * 2))
	
		local tmp = group:FindByNameSprite("bottomLeft")
		tmp:SetPosition(Vector2D(0, size.y - borderSize.y))
	
		local tmp = group:FindByNameSprite("bottomLine")
		tmp:SetPosition(Vector2D(borderSize.x, size.y - hLineHeight))
		tmp:SetSize(Vector2D(size.x - borderSize.x * 2, hLineHeight))
	
		local tmp = group:FindByNameSprite("bottomRight")
		tmp:SetPosition(Vector2D(size.x - borderSize.x, size.y - borderSize.y))
	
		local tmp = group:FindByNameSprite("rightLine")
		tmp:SetPosition(Vector2D(size.x - vLineWidth, borderSize.y))
		tmp:SetSize(Vector2D(vLineWidth, size.y - borderSize.y * 2))
	
		local tmp = group:FindByNameSprite("topRight")
		tmp:SetPosition(Vector2D(size.x - borderSize.x, 0))
	
		local tmp = group:FindByNameSprite("topLine")
		tmp:SetPosition(Vector2D(borderSize.x, 0))
		tmp:SetSize(Vector2D(size.x - borderSize.x * 2, hLineHeight))
	end
	
	-- Background
	local tmp = group:FindByName("background")
	
	local p1 = Vector2D(0, 0)
	local rectSize = size
	if group:GetUserData("isBorder") == true then
		p1 = Vector2D(vLineWidth, hLineHeight)
		rectSize = rectSize - Vector2D(vLineWidth, hLineHeight) * 2
	end
	--tmp:MakeRectangle(Vector2D(0, 0), Vector2D(size.x, size.y), Color(0, 0, 0, 140), 0, Color(0, 0, 0))
	tmp:SetPosition(p1)
	tmp:SetSize(rectSize)
	
	-- Window zone
	local tmp = group:FindByNamePhRect("windowZone")
	tmp:SetSize(size)
	
	-- Scroll strips
	--[[globalPos = group:GetGlobalPosition()

	local tmp = group:FindByNameShape("stripTop")
	tmp:MakeRectangle(Vector2D(0, -globalPos.y), Vector2D(size.x, stripMargin), Color(0, 0, 0), 0, Color(0, 0, 0))

	local tmp = group:FindByNameShape("stripBottom")
	local screenHeight = group:GetParentScreen():GetScreenConfig():GetScreenSize().y
	tmp:MakeRectangle(Vector2D(0, size.y - stripMargin), Vector2D(size.x, screenHeight - globalPos.y), Color(0, 0, 0), 0, Color(0, 0, 0))]]
	
	-- Scrollbar	
	if isScroll == true then
		local arrowUpSpr = group:FindByNameSprite("arrowUp")
		arrowUpSpr:Show(true)
		arrowUpSpr:SetPosition(Vector2D(size.x - stripMargin - arrowUpTex:GetWidth(), stripMargin))

		local tmpZone = group:FindByNamePhRect("arrowUpZone")
		tmpZone:SetPosition(arrowUpSpr:GetPosition())

		local arrowDownSpr = group:FindByNameSprite("arrowDown")
		arrowDownSpr:Show(true)
		arrowDownSpr:SetPosition(Vector2D(size.x - stripMargin - arrowDownTex:GetWidth(), size.y - stripMargin - arrowDownTex:GetHeight()))
		
		local tmpZone = group:FindByNamePhRect("arrowDownZone")
		tmpZone:SetPosition(arrowDownSpr:GetPosition())
	
		local tmp = group:FindByNameSprite("scrollBar")
		tmp:Show(true)
		tmp:SetPosition(Vector2D(size.x - stripMargin - arrowDownTex:GetWidth() / 2 - scrollBarTex:GetWidth() / 2, 0))
		
		local tmp = group:FindByNameShape("scrollBarBack")
		tmp:Show(true)
		local width = 16
		tmp:MakeRectangle(Vector2D(arrowUpSpr:GetPosition().x + arrowUpSpr:GetSize().x / 2 - width / 2, arrowUpSpr:GetPosition().y + arrowUpSpr:GetSize().y - 6), Vector2D(arrowDownSpr:GetPosition().x + arrowDownSpr:GetSize().x / 2 + width / 2, arrowDownSpr:GetPosition().y + 6), Color(255, 255, 255), 0, Color(0, 0, 0))
		
		local tmp = group:FindByNameSprite("scrollBarStart")
		tmp:Show(true)
		tmp:SetPosition(Vector2D(size.x - stripMargin - arrowUpTex:GetWidth(), stripMargin + arrowUpTex:GetWidth() + 2))
		
		local tmp = group:FindByNameSprite("scrollBarEnd")
		tmp:Show(true)
		tmp:SetPosition(Vector2D(size.x - stripMargin - arrowDownTex:GetWidth(), size.y - stripMargin - arrowDownTex:GetHeight() - tmp:GetSize().y + 3))
		
	else		
		-- Hide all
		group:FindByNameSprite("arrowUp"):Show(false)
		group:FindByNameSprite("arrowDown"):Show(false)
		group:FindByNameSprite("scrollBar"):Show(false)
		group:FindByNameShape("scrollBarBack"):Show(false)
		group:FindByNameSprite("scrollBarStart"):Show(false)
		group:FindByNameSprite("scrollBarEnd"):Show(false)		
	end
	
	-- Compute the size of the content
	local inSize = size;
	if group:GetUserData("isBorder") == true then
		inSize = Vector2D(size.x - hLost, size.y - vLost)
	end
	group:SetUserData("inSize", inSize)
	
	-- Position & scroll
	windowScroll(group, 0)
	
	-- Enable / disable look
	windowEnable(group, windowIsActive(group))
	
	-- Return the size of the content
	return inSize
end

------------------------------------------------------------
-- Set the title of a window
------------------------------------------------------------
function windowSetTitle(group, title)
	if group:GetUserData("isBorder") == true then
		local titleBar = group:FindByNameSprite("titleBar")
		local titleBarText = group:FindByNameText("titleText")
	
		-- No title => hide title bar
		if title == "" then
			titleBar:Show(false)
			titleBarText:Show(false)
		else
			titleBar:Show(true)
			titleBarText:Show(true)
		
			titleBarText:SetText(title)
		end
	end
end

------------------------------------------------------------
-- Set background colors of the window
------------------------------------------------------------
function windowSetBackgroundColors(group, colorNormal, colorActive)
	group:FindByName("background"):SetColor(colorNormal)
	--[[group:SetUserData("backColorNormal", colorNormal)
	group:SetUserData("backColorActive", colorActive)]]
end

------------------------------------------------------------
-- Enable / Disable a window
------------------------------------------------------------
function windowEnable(group, isEnabled)
	local back = group:FindByName("background")

	if group:GetUserData("isAlwaysActive") == false then		
		if isEnabled == true then
			group:SetUserData("isActive", true)
			--back:MakeRectangle(back:GetPointPosition(1), back:GetPointPosition(3), group:GetUserData("backColorActive"), 0, Color(0, 0, 0))
			back:SetImage(group:GetUserData("backColorActive"))
		else
			group:SetUserData("isActive", false)
			--back:MakeRectangle(back:GetPointPosition(1), back:GetPointPosition(3), group:GetUserData("backColorNormal"), 0, Color(0, 0, 0))
			back:SetImage(group:GetUserData("backColorNormal"))
			
			-- Set Vars
			group:SetUserData("isMoving", false)
			group:SetUserData("isScrolling", false)
		end
	else
		--back:MakeRectangle(back:GetPointPosition(1), back:GetPointPosition(3), group:GetUserData("backColorActive"), 0, Color(0, 0, 0))
		back:SetImage(group:GetUserData("backColorActive"))
	end
end

------------------------------------------------------------
-- Scroll a window
------------------------------------------------------------
function windowScroll(group, delta)
	local body = group:FindByNameGroup("body")
	
	local min = body:GetUserData("scrollMin")
	local height = body:GetUserData("scrollHeight")
	
	body:SetPositionY(body:GetPositionY() - delta) -- Delta > 0 => the content goes up
	body:SetPositionY(math.min(min, body:GetPositionY()))
	body:SetPositionY(math.max(min - height, body:GetPositionY()))
	
	local ratio = (min - body:GetPosition().y) / height
	
	local arrowUp = group:FindByNameSprite("arrowUp")
	local arrowDown = group:FindByNameSprite("arrowDown")
	
	local bar = group:FindByNameSprite("scrollBar")
	bar:SetPositionY(arrowUp:GetPosition().y + arrowUp:GetSize().y + (arrowDown:GetPosition().y - arrowUp:GetPosition().y - arrowUp:GetSize().y - bar:GetSize().y) * ratio)
	
	local tmpZone = group:FindByNamePhRect("scrollBarZone")
	tmpZone:SetPosition(bar:GetPosition())
	
	-- Scroll cut
	body:EnableCut(true)
	body:SetCutRect(Vector2D(0, min - body:GetPosition().y), group:GetUserData("inSize"))	
end

------------------------------------------------------------
-- Make a window as a top window (the one in forground)
------------------------------------------------------------
function windowMakeTop(group)
	local windowTbl = group:GetParentScreen():GetUserData("windowTbl")

	-- We disable the other window
	for i = 1, #windowTbl do
		windowEnable(windowTbl[i], false)
	end
	
	-- We activate this window
	windowEnable(group, true)
	group:Show(true)
	group:SetZpos(currentMaxZPos)
end

------------------------------------------------------------
-- Make a window as a top window (the one in forground)
------------------------------------------------------------
function windowHide(windowGrp)
	local windowTbl = windowGrp:GetParentScreen():GetUserData("windowTbl")

	-- Set Vars
	windowGrp:SetUserData("isMoving", false)
	windowGrp:SetUserData("isScrolling", false)

	-- We hide the widow
	windowGrp:Hide()

	-- We find the window under this one
	local maxId = -1
	
	for i = 1, #windowTbl do
		if (maxId == -1 or windowTbl[i]:IsUpperThan(windowTbl[maxId])) and windowIsVisible(windowTbl[i]) and not windowIsAlwaysActive(windowTbl[i]) then
			maxId = i
		end
	end
	
	-- If there are other window
	if maxId ~= -1 then
		-- We make the window under this one the top window
		windowMakeTop(windowTbl[maxId])
	else
		-- No other window => we just hide this one
		windowEnable(windowGrp, false)
	end	
end

------------------------------------------------------------
-- Return true if the window is active and visible, else otherwise
------------------------------------------------------------
function windowIsActive(windowGrp)
	return (windowGrp:GetUserData("isActive") == true or windowIsAlwaysActive(windowGrp)) and windowIsVisible(windowGrp)
	
	--[[if windowIsVisible(windowGrp) then
		if windowIsAlwaysActive(windowGrp)) then
			-- Window always active => We check if there are 
		else
			return windowGrp:GetUserData("isActive")
		end
	else
		return false
	end]]
end

------------------------------------------------------------
-- Return true if the window is visible
------------------------------------------------------------
function windowIsVisible(windowGrp)
	return windowGrp:IsVisible()
end

------------------------------------------------------------
-- Make a window always active / has always focus
------------------------------------------------------------
function windowMakeAlwaysActive(windowGrp, alwaysVisible)
	if alwaysVisible then
		windowGrp:SetUserData("isAlwaysActive", true)
		windowGrp:SetZpos(currentMaxZPos + 1)
	else
		windowGrp:SetUserData("isAlwaysActive", false)
	end
end

function windowIsAlwaysActive(windowGrp, alwaysVisible)
	return windowGrp:GetUserData("isAlwaysActive") == true
end

------------------------------------------------------------
-- Event functions
------------------------------------------------------------
-- On frame
function windowApi_OnFrame(frameTime, event)
	local windowGrp = event:GetUserData("window")

	if windowIsActive(windowGrp) then
		if IsKeyPressed("Down") then
			windowScroll(windowGrp, frameTime * 500)
		elseif IsKeyPressed("Up") then
			windowScroll(windowGrp, frameTime * -500)
		end
	
		if IsKeyPressed("MouseLeft") then
			-- If we click on an arrow
			if windowGrp:FindByNamePhRect("arrowDownZone"):IsPointInside(GetMousePosition()) then
				windowScroll(windowGrp, frameTime * 500)
			elseif windowGrp:FindByNamePhRect("arrowUpZone"):IsPointInside(GetMousePosition()) then
				windowScroll(windowGrp, frameTime * -500)
			end
		end
	end
end

-- Event wheel down
function windowApi_OnWheel(keyName, isPressed, event)
	local windowGrp = event:GetUserData("window")

	if windowIsActive(windowGrp) then
		local delta = 40
		if keyName == "MouseWheelUp" then
			delta = -delta
		end

		windowScroll(windowGrp, delta)
	end
end

-- Click on scroll bar
function windowApi_OnScrollBarClick(keyName, pressed, zoneEvent, event)
	local windowGrp = zoneEvent:GetZone():GetParent()

	if windowIsActive(windowGrp) then
		zoneEvent:GetZone():SetUserData("isScrolling", true)
		zoneEvent:GetZone():SetUserData("lastYPos", GetMousePosition().y)
	end
end

-- Click in the window
function windowApi_OnClickInWindow(keyName, pressed, zoneEvent, event)
	local windowGrp = zoneEvent:GetZone():GetParent()

	-- Visible ?
	if windowIsVisible(windowGrp) and not windowIsActive(windowGrp) then -- On ne fait qqch que si la fenêtre n'est pas encore active...
		-- We check if there is no other window on this window with a higher lowZpos
		if not windowIsPointHidden(windowGrp, GetMousePosition()) then
			-- We are the most higher window in which the user have clicked
			if sndChoiceChange ~= nil then
				windowGrp:GetParentScreen():PlaySound(sndChoiceChange)
			end
			windowMakeTop(windowGrp)
		end
	end
end

-- Click on move zone
function windowApi_OnMoveWindow(keyName, pressed, zoneEvent, event)
	local windowGrp = zoneEvent:GetZone():GetParent()

	-- Make to if we have to
	-- Visible ?
	if windowIsVisible(windowGrp) and not windowIsActive(windowGrp) then -- On ne fait qqch que si la fenêtre n'est pas encore active...
		-- We check if there is no other window on this window with a higher lowZpos
		if not windowIsPointHidden(windowGrp, GetMousePosition()) then
			-- We are the most higher window in which the user have 
			if sndChoiceChange ~= nil then
				windowGrp:GetParentScreen():PlaySound(sndChoiceChange)
			end
			windowMakeTop(windowGrp)
		end
	end

	-- Move if we can
	if windowIsActive(windowGrp) then
		windowGrp:SetUserData("isMoving", true)
		local mousePos = GetMousePosition()
		local globalPos = windowGrp:GetGlobalPosition()
		windowGrp:SetUserData("moveXPos", mousePos.x - globalPos.x)
		windowGrp:SetUserData("moveYPos", mousePos.y - globalPos.y)
	end
end

-- Click on close bt
function windowApi_OnCloseClick(button)
	local windowGrp = button:GetParent()

	-- Close the window
	windowHide(windowGrp)
end

-- Mouse move
function windowApi_OnMouseMove(pos, event)
	local windowGrp = event:GetUserData("window")

	if windowIsActive(windowGrp) then
	
		local tmpZone = windowGrp:FindByNamePhRect("scrollBarZone")
		if tmpZone:GetUserData("isScrolling") == true then		
			-- Scroll bar height
			local arrowUp = windowGrp:FindByNameSprite("arrowUp")
			local arrowDown = windowGrp:FindByNameSprite("arrowDown")
		
			if GetMousePosition().y >= (arrowUp:GetGlobalPosition().y + arrowUp:GetSize().y) and GetMousePosition().y <= arrowDown:GetGlobalPosition().y then -- If the mouse if between the two arrows
				local delta = GetMousePosition().y - tmpZone:GetUserData("lastYPos")
			
				local bar = windowGrp:FindByNameSprite("scrollBar")
				scrollBarHeight = arrowDown:GetPosition().y - arrowUp:GetPosition().y - arrowUp:GetSize().y - bar:GetSize().y
				local body = windowGrp:FindByNameGroup("body")
				local contentHeight = body:GetUserData("scrollHeight")
		
				windowScroll(windowGrp, delta * contentHeight / scrollBarHeight)
			end
		
			tmpZone:SetUserData("lastYPos", GetMousePosition().y)
		end
		
		-- Move the window
		if windowGrp:GetUserData("isMoving") == true then
			local mousePos = GetMousePosition()
			
			-- Screen
			local screen = windowGrp:GetParentScreen()
			local screenSize = screen:GetScreenConfig():GetScreenSize()
			local screenPos = screen:GetPosition()
			
			-- We are in the screen
			if (mousePos.x > screenPos.x) and (mousePos.x < screenPos.x + screenSize.x) and (mousePos.y > screenPos.y) and (mousePos.y < screenPos.y + screenSize.y) then
				-- The mouse is not hidden by a top window
				if not windowIsPointHidden(windowGrp, mousePos) then
					windowGrp:SetGlobalPosition(mousePos - Vector2D(windowGrp:GetUserData("moveXPos"), windowGrp:GetUserData("moveYPos")))
				end
			end
		end
	end
end

-- Event mouse release
function windowApi_OnMouseRelease(keyName, isPressed, event)
	local windowGrp = event:GetUserData("window")

	if windowIsActive(windowGrp) then
		event:GetUserData("window"):FindByNamePhRect("scrollBarZone"):SetUserData("isScrolling", false)
		event:GetUserData("window"):SetUserData("isMoving", false)
	end
end

