require("guiLib/guiLib")

-- Create screen
scoreScreen = CreateScreen("scoreScreen", ScreenConfig(screenSize))

-- Game on background
local tmp = scoreScreen:CreateScreenRenderer("gameRenderer")
tmp:SetTargetScreen(gameScreen)

-- Blur
local tmp = scoreScreen:CreatePostFX("blurEffect")
tmp:LoadFromFile(GetCommonDataPath() .. "/blur.sfx")

-- Color rect
scoreScreen:CreateShapeRectangle("whiteRect", Vector2D(0, 0), screenSize, Color(255, 255, 255, 100), 0, Color(0, 0, 0))

scoreNb = 10

-- Manage score
function loadScoreTbl()
	local root = GetModeConf():AccessNode("scores")
	
	tbl = {}
	for i = 1, scoreNb do
		if root:ChildExists("score" .. tostring(i)) then
			local node = root:AccessNode("score" .. tostring(i))
			
			table.insert(tbl, {})
			tbl[#tbl]["points"] = node:GetNumber("points")
			tbl[#tbl]["level"] = node:GetNumber("level")
			tbl[#tbl]["year"] = node:GetNumber("year")
			tbl[#tbl]["month"] = node:GetNumber("month")
			tbl[#tbl]["day"] = node:GetNumber("day")
		end
	end
	
	return tbl
end

function saveScoreTbl(tbl)
	-- Clear
	if GetModeConf():ChildExists("scores") then
		GetModeConf():Remove("scores")
	end
	
	local root = GetModeConf():AccessNode("scores")
	
	for i = 1, #tbl do
		local node = root:AccessNode("score" .. tostring(i))
		
		node:SetValue("points", tbl[i]["points"])
		node:SetValue("level", tbl[i]["level"])
		node:SetValue("year", tbl[i]["year"])
		node:SetValue("month", tbl[i]["month"])
		node:SetValue("day", tbl[i]["day"])
	end
end


-- Add a score to the tbl
-- Return the position where the score was added, -1 if no inserted.
function scoreAdd(points, level, tbl)
	local scorePos = -1

	for i = 1, #tbl do
		local curPoints = tbl[i]["points"]
		
		if curPoints < points then -- On arrive au score supérieur => on insert
			scorePos = i
			break
		end
	end
	
	-- Pas de place dans les score mais il reste de la place dans le tableau
	if scorePos == -1 and #tbl < scoreNb then
		scorePos = #tbl + 1
	end
	
	if scorePos ~= -1 and scorePos <= scoreNb then
		-- Insert
		table.insert(tbl, scorePos, {})
		
		tbl[scorePos]["points"] = points
		tbl[scorePos]["level"] = level
		
		temp = os.date("*t")
		tbl[scorePos]["year"] = temp["year"]
		tbl[scorePos]["month"] = temp["month"]
		tbl[scorePos]["day"] = temp["day"]
		
		-- remove  last
		while #tbl > scoreNb do
			table.remove(tbl, #tbl)
		end
	else
		scorePos = -1
	end
	
	return scorePos
end

function getScoreStringTbl(tbl, i)
	strTbl = {}
	
	strTbl.points = tostring(tbl[i]["points"])
	strTbl.level = tostring(tbl[i]["level"])
	strTbl.date = tostring(os.date("%x"), os.time(tbl[i]))
	
	return strTbl
end

-- Draw the screen
local top = 200
local btSize = texMenuBtNormal:GetSize()

-- The menu
local bt = scoreScreen:CreateGroup("backBt")
buttonControlCreate(bt, _("Back"), btSize)
buttonSetEventFunction(bt, function() ChangeScreen(menuScreen); end)
buttonSetFont(bt, 34, Color(0, 0, 0))
buttonSetImagePack(bt, texMenuBtNormal, texMenuBtOver, texMenuBtPressed)
bt:SetPosition(Vector2D(screenSize.x / 2 - btSize.x - 15, top))

local bt = scoreScreen:CreateGroup("resetBt")
buttonControlCreate(bt, _("Reset"), btSize)
buttonSetEventFunction(bt, function() popupDialog(scoreScreen, "dialogYesNo", _("Do you really want remove all scores ?"), function() scoreTbl = {}; RefreshScores(scoreTbl); saveScoreTbl(scoreTbl); end, function() end); end)
buttonSetFont(bt, 34, Color(0, 0, 0))
buttonSetImagePack(bt, texMenuBtNormal, texMenuBtOver, texMenuBtPressed)
bt:SetPosition(Vector2D(screenSize.x / 2  + 15, top))

-- Score texts
local scoreGrp = scoreScreen:CreateGroup("scoreGrp")
scoreGrp:SetPosition(Vector2D(screenSize.x / 2 - btSize.x, top + btSize.y + 50))

function createScoreTxt(col, line, text, highlight)
	local pos = Vector2D(0, 0)

	pos.x = (col - 1) * 180
	pos.y = line * 35

	local tmp = scoreGrp:CreateText("txt", text)
	tmp:SetColor(Color(0, 0, 0))
	tmp:SetSize(30)
	tmp:SetPosition(pos)
	
	if highlight == true then
		tmp:SetColor(Color(255, 0, 0))
	end
end

function RefreshScores(tbl, newScorePos)
	scoreGrp:DestroyChildren()

	createScoreTxt(1, 0, _("Position"))
	createScoreTxt(2, 0, _("Score"))
	createScoreTxt(3, 0, _("Level"))
	createScoreTxt(4, 0, _("Date"))

	for i = 1, #tbl do
		local highlight = (newScorePos == i)
		local strTbl = getScoreStringTbl(tbl, i)
		
		createScoreTxt(1, i + 1, tostring(i), highlight)
		createScoreTxt(2, i + 1, strTbl.points, highlight)
		createScoreTxt(3, i + 1, strTbl.level, highlight)
		createScoreTxt(4, i + 1, strTbl.date, highlight)
	end
end

scoreTbl = loadScoreTbl()
RefreshScores(scoreTbl, -1)

-- Event go back the menu
scoreScreen:RegisterButtonEvent(function() ChangeScreen(menuScreen); end, "Escape", true)
