------------------------------------------------------------
-- Missile 1
------------------------------------------------------------

function nuclearShell_Create(toGroup, forcedPos, enemyLevel)
	if forcedPos == nil then
		forcedPos = Vector2D(math.random() * screenSize.x, 0)
	end
	
	local enemy = stockNuclearShell1:CloneToGroup(toGroup)
	enemy:SetPosition(forcedPos)
	
	-- Speed
	enemy:SetSpeed(enemy:GetSpeed() * (0.8 + math.random() * 0.4)) -- Speed +- 40%
	
	-- Level
	local texTbl = nuclearShellTexTbl;
	
	enemyLevel = math.min(enemyLevel, #texTbl)
	enemyLevel = math.max(enemyLevel, 1)
	
	enemy:FindByNameSprite("spr"):SetImage(texTbl[enemyLevel])
	enemy:SetUserData("damage", 5 + 5 * enemyLevel)
	enemy:SetUserData("points", enemyLevel)
	
	-- Return
	return enemy
end

function nuclearShell_OnHit(phBullet, phEnemy, hitPos)
	enemyGrp = phEnemy:GetParent()
	bulletGrp = phBullet:GetParent()
	
	-- Explosion
	CreateExplosion(hitPos, math.pi * 3/5, bulletGrp:GetSpeed())

	-- Play Snd
	gameScreen:PlaySound(sndEnemyBurstTbl[math.ceil(#sndEnemyBurstTbl * math.random())])
	
	-- Points
	score = score + enemyGrp:GetUserData("points")
	
	enemyGrp:Destroy() -- We destroy the group
	
	if phBullet:GetName() ~= "phLaser" then
		phBullet:GetParent():Destroy() -- We destroy the group
	end
end

function nuclearShell_OnShieldShock(phShield, phEnemy, hitPos)
	enemyGrp = phEnemy:GetParent()
	
	-- Damage
	ChangeLife(- enemyGrp:GetUserData("damage"))
	CreateShieldImpact(hitPos.x)
	
	-- Explosion
	CreateExplosion(hitPos, math.pi, Vector2D(0, -1))
	gameScreen:PlaySound(sndShieldResist)
	
	-- We destroy the group
	enemyGrp:Destroy()
end

-- Register
addToSpawnList(function(toGroup, forcedPos) nuclearShell_Create(toGroup, forcedPos, 1); end, 1, 0.2)
addToSpawnList(function(toGroup, forcedPos) nuclearShell_Create(toGroup, forcedPos, 2); end, 9, 0.5)
addToSpawnList(function(toGroup, forcedPos) nuclearShell_Create(toGroup, forcedPos, 3); end, 15, 1)

-- Missile
stockNuclearShell1 = stockScreen:CreateGroup("nuclearShell")
stockNuclearShell1:SetSpeed(Vector2D(0, 200))
stockNuclearShell1:SetUserData("enemyType", "nuclearShell")

-- Functions
stockNuclearShell1:SetUserData("OnHitFn", nuclearShell_OnHit)
stockNuclearShell1:SetUserData("OnShieldShockFn", nuclearShell_OnShieldShock)

-- Sprite
tmpSprite = stockNuclearShell1:CreateSprite("spr")
tmpSprite:SetImage(nuclearShellTexTbl[1])
tmpSprite.y = - tmpSprite:GetSize().y / 2
tmpSprite.x = - tmpSprite:GetSize().x / 2

-- Physic
stockNuclearShell1:CreatePhCircle(phGroupEnemies, "ph", Vector2D(0, tmpSprite:GetSize().y / 2 - tmpSprite:GetSize().x / 2), tmpSprite:GetSize().x / 2)
