

/** A simple vector. */
class Vector2D
{
public:
     Vector2D(number x, number y);
     Vector2D();
    Vector2D  operator+( Vector2D vect);
    Vector2D  operator-( Vector2D vect);
    bool  operator==( Vector2D vect);
    Vector2D  operator*(number nb);
    Vector2D  operator/(number nb);
    float x;
    float y;
};

/** A color, with red, green, blue and alpha value. Min is 0 and max is 255. */
class Color
{
public:
     Color();
     Color(number r, number g, number b);
     Color(number r, number g, number b, number a);
    char r;
    char g;
    char b;
    char a;
};

/** A texture (an image) that can be loaded from a file. */
class Texture
{
public:
/** Constructor */
     Texture();
/** Return the width of the image. */
    number GetWidth();
/** Return the height of the image. */
    number GetHeight();
/** Return the size of the image. */
    Vector2D GetSize();
/** Load a texture from a file. Note: the working directory is the mode's root directory. */
    nil LoadFromFile( string Filename);
};

/** A short \a sound that can be loaded from a file. Note: a sound should not be used to load musics ( > 1 min) since the whole file is loaded into memory. */
class Sound
{
public:
/** Constructor */
     Sound();
/** Load a sound from a file. Note: the working directory is the mode's root directory. */
    nil LoadFromFile( string Filename);
};

/** A music. Note: a music is not attached to a screen. It can be played by itself. */
class Music
{
public:
/** Default constructor */
     Music();
/** Open a music file (doesn't play it -- call Play() for that)
* Note: the working directory is the mode's root directory.
* \param Filename : Path of the music file to open
* \return True if loading has been successful
*/
    nil LoadFromFile( string Filename);
/** Get the music duration
* \return Music duration, in seconds
*/
    number GetDuration();
/** Return the number of channels (1 = mono, 2 = stereo, ...)
* \return Number of channels
*/
    number GetChannelsCount();
/** Play the sound
*/
    nil Play();
/** Pause the sound
*/
    nil Pause();
/** Stop the sound
*/
    nil Stop();
/** Set the sound loop state.
* This parameter is disabled by default
* \param Loop : True to play in loop, false to play once
*/
    nil SetLoop(bool Loop);
/** Set the sound pitch.
* The default pitch is 1
* \param Pitch : New pitch
*/
    nil SetPitch(number Pitch);
/** Set the sound volume.
* The default volume is 100
* \param Volume : Volume (in range [0, 100])
*/
    nil SetVolume(number Volume);
//  */
/** Tell whether or not the sound is looping
* \return True if the sound is looping, false otherwise
*/
    bool GetLoop();
/** Get the pitch
* \return Pitch value
*/
    number GetPitch();
/** Get the volume
* \return Volume value (in range [1, 100])
*/
    number GetVolume();
/** Get the current playing position of the sound
* \return Current playing position, expressed in seconds
*/
    number GetPlayingOffset();
/** Return true if the music is playing.
* \return Current status of the sound (playing or not)
*/
    bool IsPlaying();
};

/** A text font, that can be loaded from a file. */
class Font
{
public:
/** Constructor */
     Font();
/** Load a texture from a file. Note: the working directory is the mode's root directory. */
    nil LoadFromFile( string Filename);
};


/** A class that handle arguments passed to the mode and arguments to be passed to the next mode. */
class Arguments
{
public:
/** Get the number of argument */
    number GetCount();
/** Get the next argument (or the first one if you call this function for the first time). */
    string GetNext();
/** Add an argument at the end of the list. */
    nil Add( string arg);
/** Clear all stored arguments */
    nil Clear();
};

/** A structure containing information about a mode. */
class ModeInfo
{
public:
    string fullName;
    string idName;
    string auther;
    string path;
    string description;
    string version;
    string screenshot;
};

/** A config node represent a node in the configuration file (like an XML node). You can add data to the node with a key and add child node. */
class ConfigNode
{
public:
/** Get the string corresponding to key in this node */
    string GetString( string key);
/** Get the number corresponding to key in this node */
    number GetNumber( string key);
/** Retrun true if the child key exists (it can be string / number / child node) */
    bool ChildExists( string key);
/** Set / create a string with key */
    nil SetValue( string key,  string value);
/** Set / create a number with key */
    nil SetValue( string key, number value);
/** Get the ConfigNode corresponding to key, if it exists, otherwise, create a child ConfigNode named "key" in this ConfigNode and return it */
    ConfigNode AccessNode( string key);
/** Remove the child named key */
    nil Remove( string key);
/** Clear the ConfigNode (delete all childs) */
    nil Clear();
};

/** The config manager is designed to contain the list of the mode and their information. It also contain the configuration node shared between all mode. Textdomain sould be bottle2d !*/
class ConfigManager
{
public:
/** Get information about mode idName */
    ModeInfo GetModeInfo( string idName);
/** Get information about mode number id */
    ModeInfo GetModeInfo(number id);
/** Get the current mode id */
    string GetCurrentModeId();
/** Get the root ConfigNode for the current mode */
    ConfigNode GetRootConfigNode();
/** Return the number of mode */
    number GetModeCount();
/** Get the global ConfigNode (Configuration shared between every mode) */
    ConfigNode GetGlobalConfigNode();
/** Init the locales. Path should contain the language dir (like en, fr...). Usualy, Path is "<ModePath>/local/" */
    nil InitLocales( string path);
};


/** Base event handler that can be destroyed. */
class EventHandlerBase
{
public:
    nil Destroy();
    bool ShouldBeDestroyed();
/** Set user data */
    nil SetUserData( string id,  luaObject data);
/** Get user data */
    luaObject GetUserData( string id);
/** Return true if the user data exists, false otherwise */
    bool UserDataExists( string id);
};


/** Timer event, to send event at regular interval, see Screen::RegisterTimer to register a timer event. */
class Timer : public EventHandlerBase
{
public:
/** Access the timer interval (time between two events). */
    float interval;
/** Get the timer interval (time between two events). */
    number GetInterval();
/** Set the timer interval (time between two events). */
    nil SetInterval(number numbererval);
/** Start the timer. */
    nil Start();
/** Stop / pause the timer. */
    nil Stop();
/** Return true if the timer is running, false overwise. */
    bool IsRunning();
};

/** Chronometer event, to send event on each frame while the time is not elapsed, see Screen::RegisterChronometer to register a chronometer event. */
class Chronometer : public EventHandlerBase
{
public:
/** Access the total time that the chronometer should count (when total time is elapsed, ratio is 1). */
    float totalTime;
/** Get the total time that the chronometer should count (when total time is elapsed, ratio is 1). */
    number GetTotalTime();
/** Set the total time that the chronometer should count (when total time is elapsed, ratio is 1). */
    nil SetTotalTime(number totalTime);
/** Access the current time ratio(elapsed time / total time). */
    float ratio;
/** Get the current time ratio(elapsed time / total time). */
    number GetRatio();
/** Force the chronometer to force its current ratio */
    nil SetRatio(number newRatio);
/** Restart the chronometer (set ratio and elapsed time to 0). */
    nil Restart();
/** Start the chronometer. */
    nil Start();
/** Stop / pause the chronometer. */
    nil Stop();
/** Return true if the chronometer is running, false overwise. */
    bool IsRunning();
};

/** Mouse enter / leave physic object event, see Screen::RegisterMouseZoneEvent to register a MouseZone event. */
class MouseZoneEvent : public EventHandlerBase
{
public:
/** Access the zone (a Physic object) used to detect mouse entering / exiting */
    float zone;
/** Get the zone (a Physic object) used to detect mouse entering / exiting */
    PhysicObject GetZone();
/** Set the zone (a Physic object) used to detect mouse entering / exiting */
    nil SetZone(PhysicObject zone);
/** Return true if the mouse is currently inside the zone, else overwise */
    bool IsInside();
};


/** The base of all physic and drawable object. */
class Object
{
public:
    bool  operator==( Object obj);
/** Access the object's position */
    Vector2D position;
/** Get the object's position */
    Vector2D GetPosition();
/** Set the object's position */
    nil SetPosition( Vector2D pos);
/** Access the object's rotation */
    float rotation;
/** Get the object's rotation */
    number GetRotation();
/** Set the object's rotation */
    nil SetRotation(number angle);
/** Access the object's x position */
    float x;
/** Access the object's y position */
    float y;
/** Get the object's x position */
    number GetPositionX();
/** Get the object's y position */
    number GetPositionY();
/** Set the object's x position */
    nil SetPositionX(number pos);
/** Set the object's y position */
    nil SetPositionY(number pos);
/** Access the object's global position (position from the top left corner of the screen) */
    Vector2D globalPosition;
/** Get the object's global position (position from the top left corner of the screen) */
    Vector2D GetGlobalPosition();
/** Set the object's global position (position from the top left corner of the screen) */
    nil SetGlobalPosition( Vector2D pos);
/** Access the object's global rotation (rotation of all parents and the object) */
    Vector2D rotation;
/** Get the object's global rotation (rotation of all parents and the object) */
    number GetGlobalRotation();
/** Set the object's global rotation (rotation of all parents and the object) */
    nil SetGlobalRotation(number rotation);
/** Rotate the object */
    nil Rotate(number angle);
/** Move the object */
    nil Move( Vector2D mvt);
/** Access the object's name */
    std::string name;
/** Get the object's name */
    string GetName();
/** Set the object's name */
    nil SetName( string str);
/** Access the object's speed */
    float speed;
/** Set the object's speed */
    nil SetSpeed( Vector2D speed);
/** Get the object's speed */
    Vector2D GetSpeed();
/** Access the object's angular velocity */
    float angularVelocity;
/** Set the object's angular velocity */
    nil SetAngularVelocity(number speed);
/** Get the object's angular velocity */
    number GetAngularVelocity();
/** Get the object's type id */
    number GetType();
/** Get an uniq identificator related to this object */
    long GetUniqueId();
/** Change the parent of the object */
    nil ChangeParent(Group parent);
/** Destroy the object (it will be destroyed at the end of the frame) */
    nil Destroy();
/** Get the object's parent */
    Group GetParent();
/** Clone the object */
    Object Clone();
    Object CloneToGroup(Group newParent);
/** Set user data */
    nil SetUserData( string id,  luaObject data);
/** Get user data */
    luaObject GetUserData( string id);
/** Return true if the user data exists, false otherwise */
    bool UserDataExists( string id);
/** Cast (convert) the object into its real type (you should know it). An error occures if you try to cast the object into a bad type. */
    PhLine CastIntoPhLine();
/** Cast (convert) the object into its real type (you should know it). An error occures if you try to cast the object into a bad type. */
    PhRect CastIntoPhRect();
/** Cast (convert) the object into its real type (you should know it). An error occures if you try to cast the object into a bad type. */
    PhTriangle CastIntoPhTriangle();
/** Cast (convert) the object into its real type (you should know it). An error occures if you try to cast the object into a bad type. */
    PhCircle CastIntoPhCircle();
/** Cast (convert) the object into its real type (you should know it). An error occures if you try to cast the object into a bad type. */
    PhShape CastIntoPhShape();
/** Cast (convert) the object into its real type (you should know it). An error occures if you try to cast the object into a bad type. */
    Group CastIntoGroup();
/** Cast (convert) the object into its real type (you should know it). An error occures if you try to cast the object into a bad type. */
    ParticleSource CastIntoParticleSource();
/** Cast (convert) the object into its real type (you should know it). An error occures if you try to cast the object into a bad type. */
    Shape CastIntoShape();
/** Cast (convert) the object into its real type (you should know it). An error occures if you try to cast the object into a bad type. */
    Sprite CastIntoSprite();
/** Cast (convert) the object into its real type (you should know it). An error occures if you try to cast the object into a bad type. */
    Text CastIntoText();
/** Cast (convert) the object into its real type (you should know it). An error occures if you try to cast the object into a bad type. */
    PostFX CastIntoPostFX();
/** Cast (convert) the object into its real type (you should know it). An error occures if you try to cast the object into a bad type. */
    Screen CastIntoScreen();
/** Cast (convert) the object into its real type (you should know it). An error occures if you try to cast the object into a bad type. */
    ScreenRenderer CastIntoScreenRenderer();
/** Get the screen (main parent) of this object */
    Screen GetParentScreen();
};


/** A Drawable is a base class which contains fonctions common to every Drawable object (Group, Sprite, Text...). It should not be used directly. */
class Drawable : public Object
{
public:
/** Access the object's z order (an object with a low z order will be in the background) */
    int zPos;
/** Get the object's z order */
    number GetZpos();
/** Set the object's z order (an object with a low z order will be in the background) */
    nil SetZpos(number val);
/** Get the low pos (the low zPos is a creation order, for the object that have the same zPos) */
    number GetLowZPos();
/** Hide / show the object */
    bool visible;
/** Show / hide an object */
    nil Show(bool visible = true);
/** Hide and object */
    nil Hide();
/** Return truc if the object is visible, false otherwise */
    bool IsVisible();
/** Return true if this Drawable is on to of the the given one (higher Zpos) */
    bool IsUpperThan( Drawable drawableToCompare);
};


/** A base class containing common functions to all physic objects. */
class PhysicObject : public Object
{
public:
/** Return true if the point \a is inside the object, false otherwise. */
    bool IsPointInside( Vector2D ponumber);
/** Acces the event function name (\a fnName should be the name of an existing lua function). */
    std::string eventFunction;
/** Set the event function name (\a fnName should be the name of an existing lua function).\n
* Function prototype should be : function fnName(object1 : PhysicObject, object2 : PhysicObject, intersectionPosition : Vector2D)
*/
    nil SetEventFunction( luaObject luaFunction);
/** Get the event function name. */
    luaObject GetEventFunction();
/** Get the physic group associated to the object */
    number GetPhysicGroup();
};


/** A Group is a container wich can contain drawable and physic object.
* When a group is moved or rotated, its child will be moved or rotated too. It's like a folder: if you Destroy it, its children will be destroyed too.
* It could be created using Group::CreateGroup.
*/
class Group : public Drawable
{
public:
/** Create a clone of the object and return an handler to the clone. */
    Group Clone();
/** Create a clone of the object, change the parent and return an handler to the clone. */
    Group CloneToGroup(Group newParent);
/** Edit scale */
    Vector2D scale;
/** Set the group scale. It allows you to modify the size of a single group and its childs. */
    nil SetScale( Vector2D scale);
/** Get the group scale. */
    Vector2D GetScale();
/** Return the number of physic and drawable children that this group contains */
    number GetChildCount();
/** Return the physic child number id */
    PhysicObject GetPhysicChild(number id);
/** Return the number of physic children that this group contains */
    number GetPhysicChildCount();
/** Return the drawable child number id */
    Drawable GetDrawableChild(number id);
/** Return the number of drawable children that this group contains */
    number GetDrawableChildCount();
/** Destroy every children of the group */
    nil DestroyChildren();
/** Enable / disable the cutting of a certain part of the group. Use Group::SetCutRect to define that part. */
    nil EnableCut(bool isCut);
/** Return true if the cutting is enabled, false otherwise. */
    bool IsCut();
/** Set the cut rectangle. You should enable cutting. */
    nil SetCutRect( Vector2D position,  Vector2D size);
/** Get the cut rectangle's position. */
    Vector2D GetCutRectPosition();
/** Get the cut rectangle's size. */
    Vector2D GetCutRectSize();
/*#define X(typeName, typeId)             typeName *FindByName ## typeName(const std::string &name);
X_OBJ_LIST
#undef X*/
/** Get a lua table containing all its childs.\n
* Warning: no particular order are warranted.
*/
    luaObject GetChildTable();
/** Get a child from its index in the group (a number for 1 to Group::GetChildCount().\n
* Warning: no particular order are warranted.
* Use Group::GetChildTable() to traverse a whole group : it will be faster.
*/
    luaObject FindByIndex(number id);
/** Find an object in the group with its name and cast it with the good type. Return nil if the object doesn't exist. */
    luaObject FindByName( string name);
/** Find an object in the group with its name. This function will also check the type of the object. */
    PhLine FindByNamePhLine( string name);
/** Find an object in the group with its name. This function will also check the type of the object. */
    PhRect FindByNamePhRect( string name);
/** Find an object in the group with its name. This function will also check the type of the object. */
    PhTriangle FindByNamePhTriangle( string name);
/** Find an object in the group with its name. This function will also check the type of the object. */
    PhCircle FindByNamePhCircle( string name);
/** Find an object in the group with its name. This function will also check the type of the object. */
    PhShape FindByNamePhShape( string name);
/** Find an object in the group with its name. This function will also check the type of the object. */
    Group FindByNameGroup( string name);
/** Find an object in the group with its name. This function will also check the type of the object. */
    ParticleSource FindByNameParticleSource( string name);
/** Find an object in the group with its name. This function will also check the type of the object. */
    Shape FindByNameShape( string name);
/** Find an object in the group with its name. This function will also check the type of the object. */
    Sprite FindByNameSprite( string name);
/** Find an object in the group with its name. This function will also check the type of the object. */
    Text FindByNameText( string name);
/** Find an object in the group with its name. This function will also check the type of the object. */
    PostFX FindByNamePostFX( string name);
/** Find an object in the group with its name. This function will also check the type of the object. */
    Screen FindByNameScreen( string name);
/** Find an object in the group with its name. This function will also check the type of the object. */
    ScreenRenderer FindByNameScreenRenderer( string name);
/** Create a new physic line and attach it to this group.\n
* \param phGroupId The physic group of the object, in order to generate colision events (see Screen::AddPhysicGroup()).
* \param name The name of the object.
* \param pt1 First point of the line.
* \param pt2 Second point of the line.
* \return The created object.
*/
    PhLine CreatePhLine(number phGroupId,  string name,  Vector2D pt1,  Vector2D pt2);
/** Create a new physic rectangle and attach it to this group.\n
* \param phGroupId The physic group of the object, in order to generate colision events (see Screen::AddPhysicGroup()).
* \param name The name of the object.
* \param size The size of the rectangle. Use Object::SetPosition() to change the object's position.
* \return The created object.
*/
    PhRect CreatePhRect(number phGroupId,  string name,  Vector2D size);
/** Create a new physic triangle and attach it to this group.\n
* \param phGroupId The physic group of the object, in order to generate colision events (see Screen::AddPhysicGroup()).
* \param name The name of the object.
* \param pt1 First point of the triangle.
* \param pt2 Second point of the triangle.
* \param pt3 Third point of the triangle.
* \return The created object.
*/
    PhTriangle CreatePhTriangle(number phGroupId,  string name,  Vector2D pt1,  Vector2D pt2,  Vector2D pt3);
/** Create a physic shape and attach it to this group.\n
* When a shape is created, you can add points to it by calling PhShape::AddPoint(). The shape must be convex !
* \param phGroupId The physic group of the object, in order to generate colision events (see Screen::AddPhysicGroup()).\n
* \param name The name of the object.
* \return The created object.
*/
    PhShape CreatePhShape(number phGroupId,  string name);
/** Create a new physic disk and attach it to this group.\n
* \param phGroupId The physic group of the object, in order to generate colision events (see Screen::AddPhysicGroup()).
* \param name The name of the object.
* \param center The center of the circle. Can be changed with Object::SetPosition().
* \param radius The radius of the disk.
* \return The created object.
*/
    PhCircle CreatePhCircle(number phGroupId,  string name,  Vector2D center, number radius);
/** Create a Group object and add it to the current group.\n
* \param name The name of the object.
* \return The created object.
*/
    Group CreateGroup( string name);
/** Create a ParticleSource object and add it to the current group.\n
* \param name The name of the object.
* \param model The particle model for this ParticleSource.
* \param maxParticleNumber Used to reserve memory. It should not be oversized (unusefull memory would be allocated) or undersized (some particle won't be created)
* <ul>
*     <li>If your particle source just has a flow and an unlimited tank, then \a maxParticleNumber should be \a flow * \a maxLifeTime.</li>
*     <li>If your particle is an explosion, and all particle are dead before starting a new explosion, then \a maxParticleNumber should be \a tank.</li>
* </ul>
* \return The created object.
*/
    ParticleSource CreateParticleSource( string name,  ParticleModel model, number maxParticleNumber);
/** Create a Shape object and add it to the current group.\n
* \param name The name of the object.
* \return The created object.
*/
    Shape CreateShape( string name);
/** Create a shape, initialise it as a line and add it to the current group.\n
* \param name The name of the object.
* \param P1 Position of the first point
* \param P2 Position second point
* \param Thickness Line thickness
* \param Col Color used to draw the line
* \param Outline Outline width (0 by default)
* \param OutlineCol Color used to draw the outline
* \return The created object.
*/
    Shape CreateShapeLine( string name,  Vector2D P1,  Vector2D P2, number Thickness,  Color Col, number Outline,  Color OutlineCol);
/** Create a shape, initialise it as a rectangle and add it to the current group.\n
* \param name The name of the object.
* \param P1 Position of the first point
* \param P2 Position second point
* \param Col Color used to fill the rectangle
* \param Outline Outline width
* \param OutlineCol Color used to draw the outline
* \return The created object.
*/
    Shape CreateShapeRectangle( string name,  Vector2D P1,  Vector2D P2,  Color Col, number Outline,  Color OutlineCol);
/** Create a circle / disk, initialise it as a rectangle and add it to the current group.\n
* \param name The name of the object.
* \param Center Position of the center
* \param Radius Radius
* \param Col Color used to fill the circle
* \param Outline Outline width
* \param OutlineCol Color used to draw the outline
* \return The created object.
*/
    Shape CreateShapeCircle( string name,  Vector2D Center, number Radius,  Color Col, number Outline,  Color OutlineCol);
/** Create a Sprite object and add it to the current group.\n
* \param name The name of the object.
* \return The created object.
*/
    Sprite CreateSprite( string name);
/** Create a Text object and add it to the current group.\n
* \param name The name of the object.
* \param text Text of the object.
* \return The created object.
*/
    Text CreateText( string name,  string text);
/** Create a PostFX object and add it to the current group.\n
* \param name The name of the object.
* \return The created object.
*/
    PostFX CreatePostFX( string name);
/** Create a ScreenRenderer object and add it to the current group.\n
* \param name The name of the object.
* \return The created object.
*/
    ScreenRenderer CreateScreenRenderer( string name);
    nil RegisterEventAutoRemoveOnDestroy(EventHandlerBase event);
/** Register an event to the group so that it will be destroyed when the group is destroyed.\n
* This can be useful if some event can't work without an object and can create segmentation fault when they try to use a removed object.\n
* This must be used with caution : if the event is already removed when the group is Destroyed, a segmentation fault will occure.\n
* Do not regiser one event on more than one group !
*/
};


/** A ScreenConfig is associated to a screen. It alows you to chose the screen virtual size and its ratio. */
class ScreenConfig
{
public:
/** Create a basic config with a 1024*768 size, and set according ratio */
     ScreenConfig();
/** Create a basic config with user size, and set according ratio */
     ScreenConfig( Vector2D size);
/** Set the screen size and the according ratio. Reset the camera */
    nil ChangeSize( Vector2D size);
/** Reset the camera to show the entire screen */
    nil CameraReset();
/** Access the ratio */
    float forcedRatio;
/** Set the ratio */
    nil SetForcedRatio(number value);
/** Get the ratio */
    number GetForcedRatio();
/** Choose if the engine should always use the same ratio on resize (add strips to the screen in order to keep the same ratio) */
    bool isFixedRatio;
/** Choose if the engine should always use the same ratio on resize (add strips to the screen in order to keep the same ratio) */
    nil SetFixedRatio(bool value);
/** Tells if the engine should always use the same ratio on resize */
    bool IsFixedRatio();
/** Set / Get the screen size but doesn't change the ratio */
    Vector2D screenSize;
/** Set the screen size but doesn't change the ratio */
    nil SetScreenSize( Vector2D value);
/** Get the screen size */
    Vector2D GetScreenSize();
/** Set the camera top left position */
    Vector2D cameraTopLeft;
/** Set the camera top left position */
    nil SetCameraTopLeft( Vector2D value);
/** Get the camera top left position */
    Vector2D GetCameraTopLeft();
/** Set the camera bottom right position */
    Vector2D cameraBottomRight;
/** Set the camera bottom right position */
    nil SetCameraBottomRight( Vector2D value);
/** Get the camera bottom right position */
    Vector2D GetCameraBottomRight();
/** Move the camera with the given offset */
    nil CameraMove( Vector2D mvt);
/** Zoom */
    nil CameraZoom(number addedZoom);
/** Zoom */
    nil CameraZoom(Vector2D addedZoom);
// /** Access the camera zoom */
/** Set zoom, using screen size as a reference */
    nil CameraSetZoom(number zoom);
/** Set zoom, using screen size as a reference */
    nil CameraSetZoom(Vector2D zoom);
// /** Get zoom, using screen size */
/** Access the camera half size */
    Vector2D cameraCenter;
/** Set the camera center */
    nil CameraSetCenter( Vector2D center);
/** Get the camera center */
    Vector2D CameraGetCenter();
/** Access the camera half size */
    Vector2D cameraHalfSize;
/** Set the camera half size */
    nil CameraSetHalfSize( Vector2D halfSize);
/** Get the camera half size */
    Vector2D CameraGetHalfSize();
};

/** A screen represent a part of your game where differenent physic and drawable object will evolve. You can also attach event to your screen.\n
* A bottle 2D app will always need a screen to run, but it is recomanded to use more than one. For example, you can create a menu screen and a game screen.\n
* Then, when you can switch to the game / menu by calling ChangeScreen(). When a screen is not running, it is freezed : no sound, physic and events are proceeded.\n
* Screens are created using the global function CreateScreen().
*/
class Screen : public Group
{
public:
/** Create a new button event.
* The lua function luaFunction will be called each time the button keyName is pressed (if pressed = true) or released (if pressed = false).\n
* Prototype of the called lua function : function luaFunction(keyName : string, pressedStatus : bool, thisEvent : EventHandlerBase).\n
* Note: if keyName is empty (keyName = ""), then the lua function will be called each time a button is pressed / released.\n
* Note2: The buttons could be mouse buttons, keyboard buttons and even joystick buttons.\n
* See \ref page_keyList for the list of keys.
*/
    EventHandlerBase RegisterButtonEvent( luaObject luaFunction,  string keyName, bool pressed);
/** Create a new button in zone event.
* The lua function luaFunction will be called each time the button keyName is pressed (if pressed = true) or released (if pressed = false) and the mouse is in the zone corresponding to the zoneEvt.\n
* Parameters of the called lua function : the key name (string), the pressed status (bool), the MouseZoneEvent which corresponds to the zone and the EventHandlerBase.\n
* Note: if keyName is empty (keyName = ""), then the lua function will be called each time a button is pressed / released.\n
* Note2: The buttons could be mouse buttons, keyboard buttons and even joystick buttons.\n
* See \ref page_keyList for the list of keys.
*/
    EventHandlerBase RegisterButtonInZoneEvent( luaObject luaFunction, MouseZoneEvent zoneEvt,  string keyName, bool pressed);
/** Create a new mouse move event.
* The lua function luaFunction will be called each time the mouse is moved.\n
* Parameters of the called lua function : the mouse position (Vector2D) and this EventHandlerBase.\n
* Note: if the mouse is configured as relative (cf: SetMouseRelative), then the position argument is the relative mouse mouvement.
*/
    EventHandlerBase RegisterMouseMoveEvent( luaObject luaFunction);
/**
* Create a new Timer.
* The lua function luaFunction will be called at each interval while the timer is still running.\n
* Parameters of the called lua function : this Timer.
*/
    Timer RegisterTimer( luaObject luaFunction, number numbererval, bool isRunning = true);
/** Create a new chronometer.
* The lua function luaFunction will be called at each frame during totalTime. It is  very useful to create mouvement.\n
* Parameters of the called lua function : the elapsed time ratio (float) and this Chronometer.\n
* Note: The engine ensure that the function receive a ratio of 1 at the end of the chronometer.
*/
    Chronometer RegisterChronometer( luaObject luaFunction, number totalTime, bool isRunning = true);
/** Create a new mouse in zone event.
* The lua function luaFunction will be called each time the mouse enter or leave the PhysicObject zone.\n
* Parameters of the called lua function : this MouseZoneEvent.\n
* Note : Use MouseZoneEvent::IsInside() to see if the mouse is currently inside
*/
    MouseZoneEvent RegisterMouseZoneEvent( luaObject luaFunction, PhysicObject zone);
/** Create a new Frame Event.
* The lua function luaFunction will be called at each frame with the frame time (time between the last frame) and this eventHandler as a parameter
*/
    EventHandlerBase RegisterFrameEvent( luaObject luaFunction);
/** Play a the sound. A sound is linked to a screen. If you change screen, the sound will be paused and will resume when you select back the screen. */
    nil PlaySound(Sound sound);
/** Turn on / off the mouse relative mode (cf: Screen::RegisterMouseMoveEvent) */
    nil SetMouseRelative(bool relative);
/** Tels if the mouse is in a relative or absolute mode */
    bool IsMouseRelative();
/** Show / hide the mouse cursor */
    nil ShowCursor(bool show);
/** Tels if the mouse cursor is shown */
    bool IsCursorVisible();
/** Get the number of event and pseudo event registered in this screen. */
    number GetEventCount();
// /** Show the FPS (for optimisation) */
// /** Tels if FPS ate shown */
/** Access current time speed. Elapsed time = RealTime * timeSpeed */
    float timeSpeed;
/** Set time speed */
    nil SetTimeSpeed(number timeSpeed);
/** Get time speed */
    number GetTimeSpeed();
/** Add a physic grpoup.
* To use physic intersecion events, you have to add some "physic group".\n
* Each physic object has a group id which is attributed at creation.
* The engine will only test collisions between somme registered groups. So if you want to receive event when object of group 1 intersect object from group 3, you have to call AddPhysicGroup(1, 3) or AddPhysicGroup(3, 1).\n
* Note: AddPhysicGroup(2, 2) is legal.
*/
    nil AddPhysicGroup(number phGroupId1, number phGroupId2);
/** Access the screen background color (also used for strip colors) */
    Color backgroundColor;
/** Get the screen background color (also used for strip colors) */
    Color GetBackgroundColor();
/** Set the screen background color (also used for strip colors) */
    nil SetBackgroundColor( Color color);
/** Access the screen configuration */
    ScreenConfig screenConfig;
/** Get the screen configuration */
    ScreenConfig GetScreenConfig();
/** Set the screen configuration */
    nil SetScreenConfig( ScreenConfig screenConfig);
};


/** \file header.hpp
\brief All global functions and classes.
*/
/** Select a new screen from its name */
    nil ChangeScreen( string name);
/** Select a new screen from the associated screen object */
    nil ChangeScreen(Screen screen);
/** Create a new screen named [name] and return a handler to it */
    Screen CreateScreen( string name);
/** Create a new screen named [name] and return a handler to it. This function allow you to set a ScreenConfig to this screen. */
    Screen CreateScreen( string name,  ScreenConfig screenConfig);
/** Get the path of the common data dir (data/common) */
    string GetCommonDataPath();
/** Get the path of the mode dir (data/modes/[modeName]) */
    string GetModePath();
/** Get the arguments.
* Note: if you want to pass argument to a mode, you just have to add arguments into this object.
*/
/** Get the version of the engine */
    string GetEngineVersion();
/** Recreate a render window, after changing settings */
    nil RequestRecreateRenderWindow();
/** Get the current frime time\n
* Note: 1/GetFrameTime() gives you the frame per second (FPS)
*/
    number GetFrameTime();
/** Exit the engine */
    nil Quit();
/** Convert a ratio (between 0 and 1) to make it grow faster and faster (from 0 to 0.5) and then slower and slower (from 0.5 to 1).\n
* Note: This function ensure that ConvertRatioCosAccelerationDeceleration(0) = 0 and ConvertRatioCosAccelerationDeceleration(1) = 1.
*/
    number ConvertRatioCosAccelerationDeceleration(number ratio);
/** Convert a ratio (between 0 and 1) to make it grow faster and faster (exponential type).\n
* You have to give an acceletation coefficient (> 0).\n
* Note: This function ensure that ConvertRatioAcceleration(0) = 0 and ConvertRatioAcceleration(1) = 1.
*/
    number ConvertRatioAcceleration(number ratio, number coefficient);
/** Convert a ratio (between 0 and 1) to make it grow slower and slower (exponential type).\n
* You have to give an acceletation coefficient (> 0).\n
* Note: This function ensure that ConvertRatioDeceleration(0) = 0 and ConvertRatioDeceleration(1) = 1.
*/
    number ConvertRatioDeceleration(number ratio, number coefficient);
/** Return the current mouse position */
    Vector2D GetMousePosition();
/** Return the configuration node attached to the currently runing mode (your mode) */
    ConfigNode GetModeConf();
/** Return the informations (ModeInfo) attached to the currently runing mode (your mode) */
    ModeInfo GetModeInfo();
/** Laod the mode idName (idName is the name of the mode directory, in data/modes/ or the user config directory */
    nil LoadMode( string idName);
/** Get the ConfigManager of the engine */
    ConfigManager GetConfigManager();
/** Force the engine to draw a frame. Usefull for progress bar, by example */
    nil ForceFrame();
/** Show or hide the information console */
    nil ShowConsole(bool isVisible);
/** Return true if the information console is visible, false if not */
    bool IsConsoleVisible();
/** Return true if the key \a keyName is pressed, false otherwise. See \ref page_keyList for the list of keys. */
    bool IsKeyPressed( string keyName);
/** Return the real size of the window (resolution). This function should not be used unless you want to know the real window ratio.\n
* See ScreenConfig.
*/
    Vector2D GetRealWindowSize();
/** Set if the group centers and the physic objects should be drawn (helps for debuging). */
    nil SetShowPhysicObjects(bool isVisible);
/** Return true if the group centers and the physic objects should be drawn, else otherwise. */
    bool IsShowPhysicObjects();
/** Translation function. Use Gettext to translate your string. See ConfigManager::InitLocales(). */
    string _( string str);
/** Return the type name (string) of the given type id */
    string GetTypeName(number typeId);
/** Get the child tree (for debuging purpose) of the currently running screen. */
    string GetChildTree();
/** Get the avaible video modes (return a table of Vector2D).\n
*  Note: the first element of the table is Vector2D(-1, -1), it means the mode is automaticaly chosen a the start of the engine.
*/
    luaObject GetAvaibleVideoModes();
/** Get the arguments given to bottle2d when executed (return a table of string). */
    luaObject GetCmdArguments();




/** A particle model defines the variable of the particle. The more numerous the variable are, the bigger memory size will be used by the ParticleSource. */
class ParticleModel
{
public:
/** Constructor */
     ParticleModel();
/** Constructor */
     ParticleModel(number lifeTimeMin, number lifeTimeMax);
/** Add a fixed value for parameter "name".\n
* Note: Name can be either "red", "green", "blue" or "alpha".
*/
    nil AddFixedParam( string name, number value);
/** Add a mutable value for parameter "name". This value will change from start to end during the particle life time.\n
* Note: Name can be either "red", "green", "blue" or "alpha".
*/
    nil AddMutableParam( string name, number begin, number end);
/** Add a random value for parameter "name". For each particle a value will be chosen between min and max.\n
* Note: Name can be either "red", "green", "blue" or "alpha".
*/
    nil AddRandomParam( string name, number min, number max);
/** Add a random and mutable value for parameter "name". For each particle a value will be chosen between begin_min and begin_max. At end of life, the value will be between end_min and end_max.\n
* Note: Name can be either "red", "green", "blue" or "alpha".
*/
    nil AddMutableRandomParam( string name, number begin_min, number begin_max, number end_min, number end_max);
/** Set the particle life time. A random value will be chosen between min and max for each particle. */
    nil SetLifeTime(number min, number max);
};

/** A particle emitter defines how the particle will be emitted (their direction and emission zone). */
class ParticleEmitter
{
public:
/** Default contructor */
     ParticleEmitter();
/** Set throw angles */
    nil SetAngles(number angleA, number angleB);
/** Set direction (doesn't need to be normalised) */
    nil SetDirection( Vector2D direction);
/** Make the emitter as a line */
    nil MakeLine( Vector2D pt1,  Vector2D pt2);
/** Make the emitter as a disk */
    nil MakeDisk( Vector2D center, number radius);
/** Make the emitter as a circle */
    nil MakeCircle( Vector2D center, number radius);
/** Make the emitter as a point */
    nil MakePoint( Vector2D position);
};

/** A particle source is used to send particle and create beautiful effects. It can emit only one type of particle.
* See Group::CreateParticleSource to create a particle source object.
*/
class ParticleSource : public Drawable
{
public:
/** Create a clone of the object and return an handler to the clone. */
    ParticleSource Clone();
/** Create a clone of the object, change the parent and return an handler to the clone. */
    ParticleSource CloneToGroup(Group newParent);
/** Access flow (see SetFlow) */
    int flow;
/** Set particle flow (number of particle emitted per second).\n
* Value -1 means unlimited.\n
* Note: You can't set Flow and tank to -1 (infinite).
*/
    nil SetFlow(number flow);
/** Get particle flow */
    number GetFlow();
/** Access tank (see SetTank) */
    int tank;
/** Set particle tank (the number of particle that the source can emit).\n
* Value -1 means unlimited. When tank is empty, you can add new particle by calling SetTank()\n
* Note: You can't set Flow and tank to -1 (infinite).
*/
    nil SetTank(number tank);
/** Get particle tank */
    number GetTank();
/** Access particle gravity */
    Vector2D gravity;
/** Set particle gravity */
    nil SetGravity( Vector2D force);
/** Get gravity */
    Vector2D GetGravity();
/** Access particle friction */
    float friction;
/** Set friction */
    nil SetFriction(number friction);
/** Get friction */
    number GetFriction();
/** Set emission force */
    nil SetForce(number min, number max);
/** Access minimum force */
    float minForce;
/** Get min emission force */
    number GetMinForce();
/** Set min emission force */
    nil SetMinForce(number minForce);
/** Access maximum force */
    float maxForce;
/** Get max emission force */
    number GetMaxForce();
/** Set max emission force */
    nil SetMaxForce(number maxForce);
/** Set throw angle of the emitter (0 : only in one direction, 2*pi : ain every direction) */
    nil SetEmitterAngle(number angle);
/** Get the throw angle of the emitter (0 : only in one direction, 2*pi : ain every direction) */
    number GetEmitterAngle();
/** Set direction of the emitter (doesn't need to be normalised) */
    nil SetEmitterDirection( Vector2D direction);
/** Set direction of the emitter (doesn't need to be normalised) */
    Vector2D GetEmitterDirection();
/** Set the particle emitter. Warning: It will remove most of the paramaters registered in the ParticleSource. */
    nil SetEmitter( ParticleEmitter emitter);
/** Make particle rendered as lines */
    nil MakeRenderLines(number length, number width);
/** Make particle rendered as points */
    nil MakeRenderPoints(number ponumberSize);
/** Make particle rendered as images */
    nil MakeRenderImages(Texture img,  Vector2D size, bool blendAdd);
/** Access auto destroy mode (see SetAutoDestroy) */
    bool autoDestroy;
/** Set auto destroy mode : when tank is 0, the ParticleSource should be automatically destroyed. Default is deactivated. */
    nil SetAutoDestroy(bool autoDestroy);
/** Check if auto destroy mode is activated */
    bool IsAutoDestroy();
/** Conpute particle position as if \a time seconds were elapsed. This is usefull if you to make the user think that the particle system was already running for a long time.\n
* \param time Time which is supposed to be elapsed. Usualy, it should be \a maxLifeTime.
*/
    nil ComputeParticles(number time);
/** Acces the event function (\a fnName should be an lua function or nil for nothing) that is called when the tank is empty. */
    luabind::object onTankEmptyEventFunction;
/** Set the event function (\a fnName should be an existing lua function, or nil for nothing) that is called when the tank is empty.\n
* Function prototype should be : function fnName(thisParticleSource : ParticleSource)
*/
    nil SetOnTankEmptyEventFunction( luaObject luaFunction);
/** Get the event function that is called when the tank is empty. */
    luaObject GetOnTankEmptyEventFunction();
};


/** A physic object representing a circle / disk created using Group::CreatePhCircle. */
class PhCircle : public PhysicObject
{
public:
/** Access circle's radius */
    float radius;
/** Set circle's radius */
    nil SetRadius(number radius);
/** Get circle's radius */
    number GetRadius();
/** Create a clone of the object and return an handler to the clone. */
    PhCircle Clone();
/** Create a clone of the object, change the parent and return an handler to the clone. */
    PhCircle CloneToGroup(Group newParent);
};


/** A physic object representing a line created using Group::CreatePhLine. */
class PhLine : public PhysicObject
{
public:
/** Access the line vector */
    sf::Vecor2f vector;
/** Set the line vector */
    nil SetVector( Vector2D vector);
/** Get the line vector */
    Vector2D GetVector();
/** Access the line's second point (the first one is the position) */
    sf::Vecor2f pt2;
/** Set the line's second point (the first one is the position) */
    nil SetPt2( Vector2D pt2);
/** Get the line's second point (the first one is the position) */
    Vector2D GetPt2();
/** Create a clone of the object and return an handler to the clone. */
    PhLine Clone();
/** Create a clone of the object, change the parent and return an handler to the clone. */
    PhLine CloneToGroup(Group newParent);
};


/** A physic object representing a rectangle created using Group::CreatePhRect. */
class PhRect : public PhysicObject
{
public:
/** Access the size of the rectangle. */
    Vector2D size;
/** Set the size of the rectangle. */
    Vector2D GetSize();
/** Get the size of the rectangle. */
    nil SetSize( Vector2D size);
/** Access the X size of the rectangle. */
    float sizeX;
/** Access the Y size of the rectangle. */
    float sizeY;
/** Get the X size of the rectangle. */
    number GetSizeX();
/** Get the Y size of the rectangle. */
    number GetSizeY();
/** Set the X size of the rectangle. */
    nil SetSizeX(number size);
/** Set the Y size of the rectangle. */
    nil SetSizeY(number size);
/** Create a clone of the object and return an handler to the clone. */
    PhRect Clone();
/** Create a clone of the object, change the parent and return an handler to the clone. */
    PhRect CloneToGroup(Group newParent);
};


/** A physic object representing an user shape created using Group::CreatePhShape. */
class PhShape : public PhysicObject
{
public:
/** Get the point \a nb of the shape */
    Vector2D GetPoint(number nb);
/** Set the point \a nb of the shape */
    nil SetPoint(number nb,  Vector2D pt);
/** Add a point to the shape */
    nil AddPoint( Vector2D pt);
/** Remove the point \a nb from the shape */
    nil RemovePoint(number nb);
/** Create a clone of the object and return an handler to the clone. */
    PhShape Clone();
/** Create a clone of the object, change the parent and return an handler to the clone. */
    PhShape CloneToGroup(Group newParent);
};


/** A physic object representing a triangle created using Group::CreatePhTriangle. */
class PhTriangle : public PhysicObject
{
public:
/** Get the point \a nb of the triangle (0 <= nb < 3) */
    Vector2D GetPoint(number nb);
/** Set the point \a nb of the triangle (0 <= nb < 3) */
    nil SetPoint(number nb,  Vector2D pt);
/** Create a clone of the object and return an handler to the clone. */
    PhTriangle Clone();
/** Create a clone of the object, change the parent and return an handler to the clone. */
    PhTriangle CloneToGroup(Group newParent);
};



/** A post effect is like a texture shader. It is used to add effects to the final image (add blur, fade, colorize, lens effects...).
* The language used is defined by SFML.
* You should use Group::CreatePostFX to create a PostFX object.
*/
class PostFX : public Drawable
{
public:
/** Create a clone of the object and return an handler to the clone. */
    PostFX Clone();
/** Create a clone of the object, change the parent and return an handler to the clone. */
    PostFX CloneToGroup(Group newParent);
    bool LoadFromFile( string Filename);
/** Add a parameter named \a name to the PostFX. */
    nil SetParameter( string Name, number X);
/** Add a Vector2 parameter named \a name to the PostFX. */
    nil SetParameter( string Name, number X, number Y);
/** Add a Vector3 parameter named \a name to the PostFX. */
    nil SetParameter( string Name, number X, number Y, number Z);
/** Add a Vector4 parameter named \a name to the PostFX. */
    nil SetParameter( string Name, number X, number Y, number Z, number W);
/** Add a Texture parameter named \a name to the PostFX. */
    nil SetTexture( string Name, Texture Texture);
/** Return true if the current system can use PostFX. */
    bool static CanUsePostFX();
};


/** A Shape is a basic shape with a color and an outline color. You can add and remove as much point as you want to make every shape you want.\n
* Use the function Group::CreateShape to create an empty shape.\n
* The following functions can be used to create basic shapes:
* - Group::CreateShapeLine
* - Group::CreateShapeRectangle
* - Group::CreateShapeCircle
*/
class Shape : public Drawable
{
public:
/** Create a clone of the object and return an handler to the clone. */
    Shape Clone();
/** Create a clone of the object, change the parent and return an handler to the clone. */
    Shape CloneToGroup(Group newParent);
/** Acces object's color. */
    Color color;
/** Set object's color. */
    nil SetColor( Color Col);
/** Get object's color. */
    Color GetColor();
/** Set object's scale */
    nil SetScale( Vector2D Scale);
/** Get object's scale */
    Vector2D GetScale();
/** Set object's scale (x) */
    nil SetScaleX(number FactorX);
/** Set object's scale (y) */
    nil SetScaleY(number FactorY);
/** Set object's center */
    nil SetCenter( Vector2D Center);
/** Get object's center */
    Vector2D GetCenter();
// /** Add a point to the shape */
/** Add a point to the shape. */
    nil AddPoint( Vector2D Position,  Color Col,  Color OutlineCol);
/** Get the total number of point in the shape. */
    number GetNbPoints();
/** Enable / disable filling the shape with a color. */
    nil EnableFill(bool Enable);
/** Enable / disable outline. */
    nil EnableOutline(bool Enable);
/** Change the position of a point. Index starts at 1. */
    nil SetPointPosition(number index,  Vector2D Position);
/** Change the color of a point. Index starts at 1. */
    nil SetPointColor(number index,  Color Col);
/** Change the outline color of a point. Index starts at 1. */
    nil SetPointOutlineColor(number index,  Color OutlineCol);
/** Change the outline width */
    nil SetOutlineWidth(number Width);
/** Get a point position. Index starts at 1. */
    Vector2D GetPointPosition(number index);
/** Get a point color. Index starts at 1. */
    Color GetPointColor(number index);
/** Get a point outline color. Index starts at 1. */
    Color GetPointOutlineColor(number index);
/** Get the width of the outline */
    number GetOutlineWidth();
/** Initialise the shape as a line.\n
* \param P1 Position of the first point
* \param P2 Position second point
* \param Thickness Line thickness
* \param Col Color used to draw the line
* \param Outline Outline width (0 by default)
* \param OutlineCol Color used to draw the outline
*/
    nil MakeLine( Vector2D P1,  Vector2D P2, number Thickness,  Color Col, number Outline,  Color OutlineCol);
/** Initialise the shape as a rectangle.\n
* \param P1 Position of the first point
* \param P2 Position second point
* \param Col Color used to fill the rectangle
* \param Outline Outline width
* \param OutlineCol Color used to draw the outline
*/
    nil MakeRectangle( Vector2D P1,  Vector2D P2,  Color Col, number Outline,  Color OutlineCol);
/** Initialise the shape as a circle / disk.\n
* \param Center Position of the center
* \param Radius Radius
* \param Col Color used to fill the circle
* \param Outline Outline width
* \param OutlineCol Color used to draw the outline
*/
    nil MakeCircle( Vector2D Center, number Radius,  Color Col, number Outline,  Color OutlineCol);
/** Change the size of a rectangle shape (for compatibility with Sprite). P1 position is kept. */
    nil SetSize(Vector2D size);
};


/** A Sprite is used to render a texture. It can be resised. To create a Shape, you should use Group::CreateShape. */
class Sprite : public Drawable
{
public:
/** Create a clone of the object and return an handler to the clone. */
    Sprite Clone();
/** Create a clone of the object, change the parent and return an handler to the clone. */
    Sprite CloneToGroup(Group newParent);
/** Change the sprite size. */
    Vector2D size;
/** Get the current size of the displayed sprite */
    Vector2D GetSize();
/** Set the current size of the displayed sprite */
    nil SetSize( Vector2D newSize);
/** Set an image that will be drawn by the sprite */
    nil SetImage( Texture Img);
/** Acces object's color. */
    Color color;
/** Set object's color. */
    nil SetColor( Color Col);
/** Get object's color. */
    Color GetColor();
/** Set object's scale */
    nil SetScale( Vector2D Scale);
/** Get object's scale */
    Vector2D GetScale();
/** Set object's scale (x) */
    nil SetScaleX(number FactorX);
/** Set object's scale (y) */
    nil SetScaleY(number FactorY);
/** Set object's center */
    nil SetCenter( Vector2D Center);
/** Get object's center */
    Vector2D GetCenter();
/** Set if the image should be flipped (X axis) */
    nil FlipX(bool Flipped);
/** Set if the image should be flipped (Y axis) */
    nil FlipY(bool Flipped);
/** Get the color of a pixel in the image */
    Color GetPixel(unsigned number X, unsigned number Y);
};


/** A Text is used to render text. Font, style and size can be changed. It is also posible to enable auto line return. It should be created using Group::CreateText. */
class Text : public Drawable
{
public:
/** Create a clone of the object and return an handler to the clone. */
    Text Clone();
/** Create a clone of the object, change the parent and return an handler to the clone. */
    Text CloneToGroup(Group newParent);
/** Access text */
    std::string text;
/** Set text */
    nil SetText( string text);
/** Get text */
    string GetText();
/** Return the size of the texte */
    Vector2D GetTextSize();
/** Access the text max width (see SetMaxWidth) */
    float maxWidth;
/** Set the maximum text width. It means that if a line width is greater than \a maxWidth, then some line return will be added. \a maxWidth <= 0 means no limit (default). */
    nil SetMaxWidth(number maxWidth);
/** Get the maximum text width */
    number GetMaxWidth();
/** Get / Set if words can be cut if a maximum width is set. Default is false. */
    bool canCutWords;
/** Set if words can be cut if a maximum width is set. Default is false. */
    nil SetCutWords(bool cutWords);
/** Return true if words can be cut. Default is false. */
    bool IsCutWords();
/** Acces object's color. */
    Color color;
/** Set object's color. */
    nil SetColor( Color Col);
/** Get object's color. */
    Color GetColor();
/** Set object's scale */
    nil SetScale( Vector2D Scale);
/** Get object's scale */
    Vector2D GetScale();
/** Set object's scale (x) */
    nil SetScaleX(number FactorX);
/** Set object's scale (y) */
    nil SetScaleY(number FactorY);
/** Set object's center */
    nil SetCenter( Vector2D Center);
/** Get object's center */
    Vector2D GetCenter();
/** Set the Font of the string */
    nil SetFont( Font numberFont);
/** Get the Font of the string */
    Font GetFont();
/** Set the text size of the string */
    nil SetSize(number size);
/** Get the text size of the string */
    number GetSize();
/** Acces bold parameter */
    bool bold;
/** Set if the Text should be rendered bold */
    nil SetBold(bool isBold);
/** Tells if the Text should be rendered bold */
    bool GetBold();
/** Acces bold parameter */
    bool italic;
/** Set if the Text should be rendered italic */
    nil SetItalic(bool isItalic);
/** Tells if the Text should be rendered italic */
    bool GetItalic();
/** Acces bold parameter */
    bool underligned;
/** Set if the Text should be rendered underligned */
    nil SetUnderligned(bool isUnderligned);
/** Tells if the Text should be rendered underligned */
    bool GetUnderligned();
/** Get the position of the character \a Index in the text. */
    Vector2D GetCharacterPos(size_t Index);
};


/** A screen renderer is used to draw the content of an other screen in the current screen. It can be usefull if you want to draw the content of your game in the menu background, for example.
* This object is created using Group::CreateScreenRendere.
*/
class ScreenRenderer : public Drawable
{
public:
/** Create a clone of the object and return an handler to the clone. */
    ScreenRenderer Clone();
/** Create a clone of the object, change the parent and return an handler to the clone. */
    ScreenRenderer CloneToGroup(Group newParent);
/** Access drawed screen */
    Screen targetScreen;
/** Set the target screen */
    nil SetTargetScreen(Screen targetScreen);
/** Get the target screen */
    Screen GetTargetScreen();
};

